-- Dungeon Tactics
-- Copyright 2012 lukas w. irwin [ lukas.w.irwin@gmail.com ]
-- This program is free software under the terms of the GNU General Public License version 3 or later.
-- map.lua

map = { ID = 0 }

-- creation
function map:new (o)
	-- print "[map] new map"
	o = o or {}
	setmetatable(o, self)
	self.__index = self

	o.position = xcore.x3dpoint( 0, 0, 0 )
	o.rotation = xcore.x3dpoint( 0, 0, 0 )
	o.nodes = {}
	o.mobs = {}

	self.floor   = 1 -- current floor
	self.count   = 0 -- object count
	self.max_item_types = 0 -- number of different items
	self.max_class_types = 0
	self.level_min = 1
	self.level_max = 1

	math.randomseed( os.time() )
	return o
end

function map:destroy()
	-- print "[map] destroy"
	-- save everything.
	-- destroy map tiles
	if self.tiles ~= nil then
		tile_count = table.getn( self.tiles )
		for x=tile_count,1,-1 do
			for y=tile_count,1,-1 do
				self.tiles[ x ][ y ]:destroy()
				table.remove ( self.tiles[ x ], y )
			end
			table.remove ( self.tiles, x )
		end
	end

	-- destroy mobs
	if self.mobs ~= nil then
		mob_count = table.getn( self.mobs )
		for i=mob_count,1,-1 do
			self.mobs[i]:destroy()
			table.remove ( self.mobs, i )
		end
	end

	-- destroy nodes
	node_count = table.getn( self.nodes )
	for i=node_count,1,-1 do
		self.nodes[i]:destroy()
		table.remove ( self.nodes, i )
	end

	if self.mobs ~= nil then
		mob_count = table.getn( self.mobs )
		for i=mob_count,1,-1 do
			self.mobs[i]:destroy()
			table.remove ( self.mobs, i )
		end
	end	
	-- print "[map] /destroy"
end

function map:create()
	-- print "[map] create"
	self:loadTileset()
	self.tiles = {}

	if map_type == TILESET_DUNGEON then -- dungeon
		self:createDungeon()
	elseif map_type == TILESET_FOREST then -- forest	
		self:createForest()
	elseif map_type == TILESET_CAVE then -- cave
		self:createCave()
	elseif map_type == TILESET_MAZE then -- maze
		self:createMaze()
	elseif map_type == TILESET_CASTLE then -- maze
		self:createCastle()
	else
		print("unknown map type")
	end
end

function map:getTile( t )
	-- print("[map] getTile")
	-- return index of type t
	local tmp = -1
	for i = 0, table.getn( self.tileset ) do
		if self.tileset[i]["type"] == t then
			if tmp == -1 then
				tmp = i
			else
				if math.random(1, 100) > 90 then
					tmp = i
				end
			end
		end
	end

	return tmp
end

function map:loadTileset()
	-- print("[map] loadTileset")
	-- get tileset data
	self.tileset = {}
	
	statement = "SELECT tile_name, tile_type, tile_model, tile_texture, tile_scale, tile_base, tile_height FROM table_tiles WHERE tile_set = " .. map_type
	-- print( statement )
	local q = sql:exe( statement )

	if q ~= nil then
		-- print ("returned " .. q:getRowCount() .. " rows ")		
		for i = 0, q:getRowCount()-1 do
			local n = i*q:getHeadCount()
			-- print("\nname : "    .. q:getData( n+0 ) )
			-- print(" type : "    .. q:getData( n+1 ) )
			-- print(" model : "   .. q:getData( n+2 ) )
			-- print(" texture : " .. q:getData( n+3 ) )
			-- print(" scale : "   .. q:getData( n+4 ) )
			-- print(" billboard : "   .. q:getData( n+5 ) )
			-- print(" height : "   .. q:getData( n+6 ) )

			self.tileset[i] = {}
			self.tileset[i]["name"] = q:getData( n+0 )
			self.tileset[i]["type"] = tonumber( q:getData( n+1 ) )
			self.tileset[i]["model"] = q:getData( n+2 )
			self.tileset[i]["texture"] = q:getData( n+3 )
			self.tileset[i]["scale"] = tonumber( q:getData( n+4 ) )
			self.tileset[i]["billboard"] = q:getData( n+5 )
			self.tileset[i]["height"] = tonumber( q:getData( n+6 ) )
		end
	else
		print("[map] Error : No tileset defined")
	end	
end

function map:createMaze()
	print("[map] createMaze")
	self:makeRoom( xcore.x2dpoint( 1, 1 ), 3, 3, math.random(1, 4) )
	-- add a bunch of halls
	for i = 1, map_size*20 do
		-- pick a wall
		self:stepMaze()
	end
end

function map:createCave()
	print("[map] createCave")
	map_feature_type = "Cave Wall"
	for x = 1, map_size do
		self.tiles[ x ] = {}     -- create a new row
		for y = 1, map_size do
			self.count = self.count + 1
			self.tiles[ x ][ y ] =  tile:new()
			self.tiles[ x ][ y ]:setID( self.count )
			self.tiles[ x ][ y ]:setType( TILE_TYPE_BLOCK ) -- TILE_TYPE_BLOCK
			self.tiles[ x ][ y ]:setText( map_feature_type )
			self.tiles[ x ][ y ]:setPosition( xcore.x3dpoint( x*tile_size, 0, y*tile_size) )
		end
	end

	-- pick centeral random tile.
	x = math.floor( map_size/2 )
	y = math.floor( map_size/2 )
	self:setCell( x, y, TILE_TYPE_CLEAR )
	map_feature_type = "Dirt"
	
	clear_tiles = 0
	-- add a clear tile to a random place that is a) adjacent to existing tile and b) not adjacent to map edge.
	for i = 0, (map_size*map_size)*map_size do
		x = math.random(2, map_size-1)
		y = math.random(2, map_size-1)
		-- print "[generate] 1"
		if self.tiles[ x ][ y ].tile_type == TILE_TYPE_BLOCK then
			-- print "[generate] 2"
		
			if clear_tiles < (map_size*map_size)- (map_size*map_size)/3 then
					if self.tiles[x-1][y].tile_type == TILE_TYPE_CLEAR then
					-- print "place tile"
					self:setCell( x, y, TILE_TYPE_CLEAR )
					clear_tiles = clear_tiles +1
				end
				if self.tiles[x+1][y].tile_type == TILE_TYPE_CLEAR then
					-- print "place tile"
					self:setCell( x, y, TILE_TYPE_CLEAR )
					clear_tiles = clear_tiles +1
				end
				if self.tiles[x][y-1].tile_type == TILE_TYPE_CLEAR then
					-- print "place tile"
					self:setCell( x, y, TILE_TYPE_CLEAR )
					clear_tiles = clear_tiles +1
				end
				if self.tiles[x][y+1].tile_type == TILE_TYPE_CLEAR then
					-- print "place tile"
					-- keep track ov how many tiles have been placed.
					self:setCell( x, y, TILE_TYPE_CLEAR )
					clear_tiles = clear_tiles +1
				end
			end
		end
		-- if placedTiles == maxLand then
			-- stop after placing enough clear tiles.
		-- 	trys = maxTrys
		-- end
	end

	-- add some stalagtites and some rocks
	for n = 1, map_size do
		map_feature_type = "rocks"
		x = math.random(2, map_size-1 )
		y = math.random(2, map_size-1 )
		if self.tiles[x][y].tile_type == TILE_TYPE_CLEAR then
			self:setCell( x, y, TILE_TYPE_IMPASS )
		end
	end
	
	-- add a start and an exit 

	-- add endpoints
	endcap[1]:setPosition( xcore.x3dpoint( 0, 0, (map_size+1)*tile_size) )
	endcap[2]:setPosition( xcore.x3dpoint( (map_size+1)*tile_size, 0, 0) )
	endcap[3]:setPosition( xcore.x3dpoint( (map_size+1)*tile_size, 0, (map_size+1)*tile_size) )
	endcap[4]:setPosition( xcore.x3dpoint( 0, 0, 0) )
end

function map:createForest()
	-- print("[map] createForest")
	map_feature_type = "grass"
	for x = 1, map_size do
		self.tiles[ x ] = {}     -- create a new row
		for y = 1, map_size do
			self.count = self.count + 1
			self.tiles[ x ][ y ] =  tile:new()
			self.tiles[ x ][ y ]:setID( self.count )
			self.tiles[ x ][ y ]:setType( TILE_TYPE_CLEAR ) -- TILE_TYPE_BLOCK
			self.tiles[ x ][ y ]:setText( map_feature_type )
			self.tiles[ x ][ y ]:setPosition( xcore.x3dpoint( x*tile_size, 0, y*tile_size) )
		end
	end
	-- add a start to a random location on the edge of the map
	loc = math.random( 3, map_size-2 )		
	dir = math.random( 1, 4 )
	if dir == 1 then
		startpoint = xcore.x2dpoint( 1, loc )
	elseif dir == 2 then
		startpoint = xcore.x2dpoint( map_size, loc )
	elseif dir == 3 then
		startpoint = xcore.x2dpoint( loc, 1 )
	elseif dir == 4 then
		startpoint = xcore.x2dpoint( loc, map_size )
	end

	-- print(" add exit")
	-- add the exit to the opposite side
	loc = math.random( 3, map_size-2 )
	if dir == 1 then
		endpoint = xcore.x2dpoint( map_size, loc )
	elseif dir == 2 then
		endpoint = xcore.x2dpoint( 1, loc )
	elseif dir == 3 then
		endpoint = xcore.x2dpoint( loc, map_size )
	elseif dir == 4 then
		endpoint = xcore.x2dpoint( loc, 1 )
	end

	-- connect the start and exit with a road
	if math.random( 1, 100 ) > 20 then
		-- print("make road ")
		map_feature_type = "a path"	
		findPath(  startpoint, endpoint ) -- stupic c style code...
		for i = 1, table.getn( pathList ) do
			loc = getTilePosByID( pathList[ i ].tileID )
			self:setCell( loc.X, loc.Y, TILE_TYPE_ROAD )
		end
		-- chance for fork in the road?
	
		-- print("dir :" ..dir  .. " shortcut @ ".. map_size/2 .. ", " .. map_size )
		-- print("math.floor( table.getn( pathList )/2 ) : " .. math.floor( table.getn( pathList )/2 ) )
		

		if dir == 1 then
			-- print("dir 1; shortcut @ ".. map_size/2 .. ", " .. map_size )
			shortcut = xcore.x2dpoint( map_size/2, map_size )
			-- shortcut = xcore.x2dpoint( 1, map_size/2 )
			map_feature_type = "a shortcut"	
			self:setCell( shortcut.X, shortcut.Y, TILE_TYPE_ROAD )
			fork = getTilePosByID( pathList[ math.floor( table.getn( pathList )/2 ) ].tileID )
			findPath(  fork, shortcut )
			map_feature_type = "a path"	
			for i = 1, table.getn( pathList ) do
				loc = getTilePosByID( pathList[ i ].tileID )
				self:setCell( loc.X, loc.Y, TILE_TYPE_ROAD )
			end
			
		elseif dir == 2 then
			
			shortcut = xcore.x2dpoint( map_size/2, 1 )
			-- shortcut = xcore.x2dpoint( 1, map_size/2 )
			map_feature_type = "a shortcut"	
			self:setCell( shortcut.X, shortcut.Y, TILE_TYPE_ROAD )
			fork = getTilePosByID( pathList[ math.floor( table.getn( pathList )/2 ) ].tileID )
			findPath(  fork, shortcut )
			map_feature_type = "a path"	
			for i = 1, table.getn( pathList ) do
				loc = getTilePosByID( pathList[ i ].tileID )
				self:setCell( loc.X, loc.Y, TILE_TYPE_ROAD )
			end
			
		elseif dir == 3 then
			-- print("dir 1; shortcut @ ".. map_size/2 .. ", " .. map_size )
			shortcut = xcore.x2dpoint( 1, map_size/2 )
			-- shortcut = xcore.x2dpoint( 1, map_size/2 )
			map_feature_type = "a shortcut"	
			self:setCell( shortcut.X, shortcut.Y, TILE_TYPE_ROAD )
			fork = getTilePosByID( pathList[ math.floor( table.getn( pathList )/2 ) ].tileID )
			findPath(  fork, shortcut )
			map_feature_type = "a path"	
			for i = 1, table.getn( pathList ) do
				loc = getTilePosByID( pathList[ i ].tileID )
				self:setCell( loc.X, loc.Y, TILE_TYPE_ROAD )
			end
			
		elseif dir == 4 then
			-- print("dir 1; shortcut @ ".. map_size/2 .. ", " .. map_size )
			shortcut = xcore.x2dpoint( map_size, map_size/2 )
			-- shortcut = xcore.x2dpoint( 1, map_size/2 )
			map_feature_type = "a shortcut"
			self:setCell( shortcut.X, shortcut.Y, TILE_TYPE_ROAD )
			fork = getTilePosByID( pathList[ math.floor( table.getn( pathList )/2 ) ].tileID )
			findPath(  fork, shortcut )
			map_feature_type = "a path"	
			for i = 1, table.getn( pathList ) do
				loc = getTilePosByID( pathList[ i ].tileID )
				self:setCell( loc.X, loc.Y, TILE_TYPE_ROAD )
			end
			
		end
		-- fork = getTilePosByID( pathList[ math.floor( table.getn( pathList )/2 ) ].tileID )
		-- findPath(  fork, shortcut )
	end

	map_feature_type = "start area"
	self:setCell( startpoint.X, startpoint.Y, TILE_TYPE_START )
	start_indicator:setPosition( xcore.x3dpoint( startpoint.X*tile_size , 10, startpoint.Y*tile_size ) )
	start_indicator:setVisible( true )
	cam_target = xcore.x2dpoint( startpoint.X, startpoint.Y )
		
	map_feature_type = "exit area"
	self:setCell( endpoint.X, endpoint.Y, TILE_TYPE_EXIT )			
	exit_indicator:setPosition( xcore.x3dpoint( endpoint.X*tile_size , 10, endpoint.Y*tile_size ) )
	exit_indicator:setVisible( true )

	-- make any tiles  that are 3 sides with road into road
	
	for x = 1, map_size do
		for y = 1, map_size do
			if self.tiles[ x ][ y ].tile_type == TILE_TYPE_CLEAR then
				local contacts = 0
				map_feature_type = "grass"
				if x > 1 then
					if self.tiles[ x-1 ][ y ].tile_type == TILE_TYPE_ROAD or self.tiles[ x-1 ][ y ].tile_type == TILE_TYPE_START or self.tiles[ x-1 ][ y ].tile_type == TILE_TYPE_EXIT then
						contacts = contacts + 1
						dn = 1
					end
				end
				if y > 1 then
					if self.tiles[ x ][ y-1 ].tile_type == TILE_TYPE_ROAD or self.tiles[ x ][ y-1 ].tile_type == TILE_TYPE_START or self.tiles[ x ][ y-1 ].tile_type == TILE_TYPE_EXIT  then
						contacts = contacts + 1
						de = 1
					end
				end
				if x < map_size then
					if self.tiles[ x+1 ][ y ].tile_type == TILE_TYPE_ROAD or self.tiles[ x+1 ][ y ].tile_type == TILE_TYPE_START or self.tiles[ x+1 ][ y ].tile_type == TILE_TYPE_EXIT then
						contacts = contacts + 1
						ds = 1
					end
				end
				if y < map_size then
					if self.tiles[ x ][ y+1 ].tile_type == TILE_TYPE_ROAD or self.tiles[ x ][ y+1 ].tile_type == TILE_TYPE_START or self.tiles[ x ][ y+1 ].tile_type == TILE_TYPE_EXIT  then
						contacts = contacts + 1
						dw = 1
					end
				end

				if contacts >= 3 then
					map_feature_type = "a path"
					self:setCell( x, y, TILE_TYPE_ROAD )
				end
			end
		end
	end
	
	-- have to fill in the edges of the road
	map_feature_type = "grass"
	for x = 1, map_size do
		for y = 1, map_size do
			if self.tiles[ x ][ y ].tile_type == TILE_TYPE_CLEAR then
				local contacts = 0
				local dn = 0
				local de = 0
				local ds = 0
				local dw = 0
				
				if x > 1 then
					if self.tiles[ x-1 ][ y ].tile_type == TILE_TYPE_ROAD or self.tiles[ x-1 ][ y ].tile_type == TILE_TYPE_START or self.tiles[ x-1 ][ y ].tile_type == TILE_TYPE_EXIT then
						contacts = contacts + 1
						dn = 1
					end
				end
				if y > 1 then
					if self.tiles[ x ][ y-1 ].tile_type == TILE_TYPE_ROAD or self.tiles[ x ][ y-1 ].tile_type == TILE_TYPE_START or self.tiles[ x ][ y-1 ].tile_type == TILE_TYPE_EXIT  then
						contacts = contacts + 1
						de = 1
					end
				end
				if x < map_size then
					if self.tiles[ x+1 ][ y ].tile_type == TILE_TYPE_ROAD or self.tiles[ x+1 ][ y ].tile_type == TILE_TYPE_START or self.tiles[ x+1 ][ y ].tile_type == TILE_TYPE_EXIT then
						contacts = contacts + 1
						ds = 1
					end
				end
				if y < map_size then
					if self.tiles[ x ][ y+1 ].tile_type == TILE_TYPE_ROAD or self.tiles[ x ][ y+1 ].tile_type == TILE_TYPE_START or self.tiles[ x ][ y+1 ].tile_type == TILE_TYPE_EXIT  then
						contacts = contacts + 1
						dw = 1
					end
				end

				if contacts > 0 then
					if dn == 1 and de == 1 then -- NE
						self:setCell( x, y, TILE_TYPE_ROAD_OUTTER )
						self.tiles[ x ][ y ]:setRotation( 4 ) -- good
					elseif ds == 1 and de == 1 then -- SE
						self:setCell( x, y, TILE_TYPE_ROAD_OUTTER )
						self.tiles[ x ][ y ]:setRotation( 2 )				
					elseif dn == 1 and dw == 1 then -- NW
						self:setCell( x, y, TILE_TYPE_ROAD_OUTTER )
						self.tiles[ x ][ y ]:setRotation( 1 )			
					elseif ds == 1 and dw == 1 then -- SW
						self:setCell( x, y, TILE_TYPE_ROAD_OUTTER )
						self.tiles[ x ][ y ]:setRotation( 3 ) -- good
					elseif dn == 1 then
						self:setCell( x, y, TILE_TYPE_ROAD_EDGE )
						self.tiles[ x ][ y ]:setRotation( 4 )
					elseif de == 1 then
						self:setCell( x, y, TILE_TYPE_ROAD_EDGE )
						self.tiles[ x ][ y ]:setRotation( 2 )				
					elseif ds == 1 then
						self:setCell( x, y, TILE_TYPE_ROAD_EDGE )
						self.tiles[ x ][ y ]:setRotation( 3 )				
					elseif dw == 1 then
						self:setCell( x, y, TILE_TYPE_ROAD_EDGE )
						self.tiles[ x ][ y ]:setRotation( 1 )					
					end
				end
			end
		end
	end

	-- final pass
	map_feature_type = "grass"
	for x = 1, map_size do
		for y = 1, map_size do
			if self.tiles[ x ][ y ].tile_type == TILE_TYPE_CLEAR then
				local contacts = 0
				local dn = 0
				local de = 0
				local ds = 0
				local dw = 0
				
				if x > 1 and y < map_size then
					if self.tiles[ x-1 ][ y+1 ].tile_type == TILE_TYPE_ROAD or self.tiles[ x-1 ][ y+1 ].tile_type == TILE_TYPE_START or self.tiles[ x-1 ][ y+1 ].tile_type == TILE_TYPE_EXIT then
						contacts = contacts + 1
						dn = 1
					end
				end

				if y > 1 and x < map_size then
					if self.tiles[ x+1 ][ y-1 ].tile_type == TILE_TYPE_ROAD or self.tiles[ x+1 ][ y-1 ].tile_type == TILE_TYPE_START or self.tiles[ x+1 ][ y-1 ].tile_type == TILE_TYPE_EXIT then
						contacts = contacts + 1
						de = 1
					end
				end
				if x < map_size and y < map_size then
					if self.tiles[ x+1 ][ y+1 ].tile_type == TILE_TYPE_ROAD or self.tiles[ x+1 ][ y+1 ].tile_type == TILE_TYPE_START or self.tiles[ x+1 ][ y+1 ].tile_type == TILE_TYPE_EXIT then
						contacts = contacts + 1
						ds = 1
					end
				end
				if y > 1 and x > 1 then
					if self.tiles[ x-1 ][ y-1 ].tile_type == TILE_TYPE_ROAD or self.tiles[ x-1 ][ y-1 ].tile_type == TILE_TYPE_START or self.tiles[ x-1 ][ y-1 ].tile_type == TILE_TYPE_EXIT then
						contacts = contacts + 1
						dw = 1
					end
				end

				if contacts > 0 then
					if dn == 1 then -- NE
						-- map_feature_type = "dn"
						self:setCell( x, y, TILE_TYPE_ROAD_INNER )
						self.tiles[ x ][ y ]:setRotation( 1 ) -- good
					elseif de == 1 then -- SE
						-- map_feature_type = "de"
						self:setCell( x, y, TILE_TYPE_ROAD_INNER )
						self.tiles[ x ][ y ]:setRotation( 2 )				
					elseif ds == 1 then -- NW
						-- map_feature_type = "ds"
						self:setCell( x, y, TILE_TYPE_ROAD_INNER )
						self.tiles[ x ][ y ]:setRotation( 3 )			
					elseif dw == 1 then -- SW
						-- map_feature_type = "dw"
						self:setCell( x, y, TILE_TYPE_ROAD_INNER )
						self.tiles[ x ][ y ]:setRotation( 4 )
					end
				end
			end
		end
	end
	
	-- add some walls

	-- add some trees
	-- print(" add trees")
	for n = 1, map_size*2 do
		map_feature_type = "a tree"
		x = math.random(2, map_size-1 )
		y = math.random(2, map_size-1 )	
		if self.tiles[ x ][ y ].tile_type == TILE_TYPE_CLEAR then
			self:setCell( x, y, TILE_TYPE_BLOCK )
		end
	end	
	-- add some rocks
	-- add some flowers
	
	-- for i = 1, map_size do
	-- 	x = math.random(1, map_size)
	-- 	y = math.random(1, map_size)
	-- 	self.tiles[ x ][ y ]:setType( TILE_TYPE_BLOCK )
	-- 	self.tiles[ x ][ y ]:setPosition( xcore.x3dpoint( x*tile_size, 0, y*tile_size) )
	-- end

	-- set endcaps
	endcap[1]:setPosition( xcore.x3dpoint( 0, -1000, 0 ) )
	endcap[2]:setPosition( xcore.x3dpoint( 0, -1000, 0 ) )
	endcap[3]:setPosition( xcore.x3dpoint( 0, -1000, 0 ) )
	endcap[4]:setPosition( xcore.x3dpoint( 0, -1000, 0 ) )
end

function map:createDungeon()
	-- print("[map] createDungeon")
	-- fill with black tiles
	map_feature_type = "nothing"
	for x = 1, map_size do
		self.tiles[ x ] = {}     -- create a new row
		for y = 1, map_size do
			self.count = self.count + 1
			self.tiles[ x ][ y ] =  tile:new()
			self.tiles[ x ][ y ]:setID( self.count )
			self.tiles[ x ][ y ]:setType( TILE_TYPE_UNUSED ) -- TILE_TYPE_BLOCK
			self.tiles[ x ][ y ]:setText( map_feature_type )
			self.tiles[ x ][ y ]:setPosition( xcore.x3dpoint( x*tile_size, 0, y*tile_size) )
		end
	end

	-- carve out the first room in center
	room_width = math.random(3,9)
	room_length = math.random(3,9)
	-- math.random(1, map_size-room_width), math.random(1, map_size-room_width)
	if self:makeRoom( xcore.x2dpoint( map_size/2, map_size/2 ), room_width, room_length, math.random(1, 4) ) == false then
		-- print("no room?")
		self:makeRoom( xcore.x2dpoint( map_size/2, map_size/2 ), 3, 3, math.random(1, 4) )
	end

	-- add rooms and halls
	for i = 1, map_size*map_size do
		-- pick a wall
		self:stepDungeon()
	end
	
	for i = 1, map_size*5 do
		-- fix dead ends
		self:removeWalls()
	end

	-- add some doors
	for i = 1, map_size do
		self:addDoors()
	end

	-- add an exit.
	if dungeon_level < map_depth then
		current_location = self:getDeadEnd()
		if current_location ~= nil then			
			self.tiles[ current_location.X ][ current_location.Y ]:setType( TILE_TYPE_EXIT )
			self.tiles[ current_location.X ][ current_location.Y ]:setText("Exit")
		else
			-- print("no dead end found, place exit in room")
			current_location = self:getStairWell()
			if current_location ~= nil then
				self.tiles[ current_location.X ][ current_location.Y ]:setType( TILE_TYPE_EXIT )
				self.tiles[ current_location.X ][ current_location.Y ]:setText("Exit")
			else
				-- print("[map] no exit.... place at random.")
				current_location = self:findFloor()
				if current_location ~= nil then
					self.tiles[ current_location.X ][ current_location.Y ]:setType( TILE_TYPE_EXIT )
					self.tiles[ current_location.X ][ current_location.Y ]:setText("Exit")
				end
			end
		end
		if current_location ~= nil then
			-- print("\n[map] set exit rotation")
			if self.tiles[ current_location.X+1 ][ current_location.Y ].tile_type == TILE_TYPE_CLEAR then
				self.tiles[ current_location.X ][ current_location.Y ]:setRotation(1)
			elseif self.tiles[ current_location.X-1 ][ current_location.Y ].tile_type == TILE_TYPE_CLEAR then
				self.tiles[ current_location.X ][ current_location.Y ]:setRotation(2)
			elseif self.tiles[ current_location.X ][ current_location.Y-1 ].tile_type == TILE_TYPE_CLEAR then
				self.tiles[ current_location.X ][ current_location.Y ]:setRotation(3)
			elseif self.tiles[ current_location.X ][ current_location.Y+1 ].tile_type == TILE_TYPE_CLEAR then
				self.tiles[ current_location.X ][ current_location.Y ]:setRotation(4)
			end
		end
		exit_indicator:setPosition( xcore.x3dpoint( current_location.X*tile_size , 20, current_location.Y*tile_size ) )
		exit_indicator:setVisible( true )		
	else
		-- add the goal?
		-- print("[map] add goal!")
	end

	-- add a start zone
	current_location = self:findRandomStart( current_location )
	if current_location ~= nil then
		self.tiles[ current_location.X ][ current_location.Y ]:setType( TILE_TYPE_START )
		-- set rotation
		cam_target = xcore.x2dpoint( current_location.X, current_location.Y )
	else
		-- print("[map] no start... try again")
		current_location = self:findStart()
		
		if current_location ~= nil then
			self.tiles[ current_location.X ][ current_location.Y ]:setType( TILE_TYPE_START )
			-- set rotation
			cam_target = xcore.x2dpoint( current_location.X, current_location.Y )
		else
			-- print("[map] abort map, clear and retry from start")
			self:clear()
			self:create()
		end
	end
	if current_location ~= nil then
		-- print("[map] set start rotation")
		if self.tiles[ current_location.X+1 ][ current_location.Y ].tile_type == TILE_TYPE_BLOCK then
			self.tiles[ current_location.X ][ current_location.Y ]:setRotation(1)
			
		elseif self.tiles[ current_location.X-1 ][ current_location.Y ].tile_type == TILE_TYPE_BLOCK then
			self.tiles[ current_location.X ][ current_location.Y ]:setRotation(2)
			
		elseif self.tiles[ current_location.X ][ current_location.Y-1 ].tile_type == TILE_TYPE_BLOCK then
			self.tiles[ current_location.X ][ current_location.Y ]:setRotation(3)
			
		elseif self.tiles[ current_location.X ][ current_location.Y+1 ].tile_type == TILE_TYPE_BLOCK then
			self.tiles[ current_location.X ][ current_location.Y ]:setRotation(4)
		end	
		
		start_indicator:setPosition( xcore.x3dpoint( current_location.X*tile_size , 20, current_location.Y*tile_size ) )
		start_indicator:setVisible( true )
	end

	-- add room accents
	for i = 1, map_size do
		self:findRoom()
	end

	endcap[1]:setPosition( xcore.x3dpoint( 0, 0, (map_size+1)*tile_size) )
	endcap[2]:setPosition( xcore.x3dpoint( (map_size+1)*tile_size, 0, 0) )
	endcap[3]:setPosition( xcore.x3dpoint( (map_size+1)*tile_size, 0, (map_size+1)*tile_size) )
	endcap[4]:setPosition( xcore.x3dpoint( 0, 0, 0) )
end

function map:createCastle()
	-- print("[map] createCastle")
	-- fill with black tiles
	map_feature_type = "nothing"
	for x = 1, map_size do
		self.tiles[ x ] = {}     -- create a new row
		for y = 1, map_size do
			self.count = self.count + 1
			self.tiles[ x ][ y ] =  tile:new()
			self.tiles[ x ][ y ]:setID( self.count )
			self.tiles[ x ][ y ]:setType( TILE_TYPE_UNUSED ) -- TILE_TYPE_BLOCK
			self.tiles[ x ][ y ]:setText( map_feature_type )
			self.tiles[ x ][ y ]:setPosition( xcore.x3dpoint( x*tile_size, 0, y*tile_size) )
		end
	end
	
	-- carve out the first room in center
	room_width = 6
	room_length = 6
	-- math.random(1, map_size-room_width), math.random(1, map_size-room_width)
	if self:makeRoom( xcore.x2dpoint( map_size/2, map_size/2 ), room_width, room_length, 1 ) == false then
		-- print("wrong direction")
	end

	-- add an exit.
	if dungeon_level < map_depth then
		self.tiles[ 7 ][ 8 ]:setType( TILE_TYPE_EXIT )
		self.tiles[ 7 ][ 8 ]:setText("Exit")
		exit_indicator:setPosition( xcore.x3dpoint( 7*tile_size , 10, 8*tile_size ) )
		exit_indicator:setVisible( true )
	end
	-- add a start zone

	self.tiles[ 11 ][ 4 ]:setType( TILE_TYPE_START )
	self.tiles[ 11 ][ 4 ]:setText("start area")

	start_indicator:setPosition( xcore.x3dpoint( 11*tile_size, 20, 4*tile_size ) )
	start_indicator:setVisible( true )

	
	-- set rotation
	cam_target = xcore.x2dpoint( 11, 4 )

	endcap[1]:setPosition( xcore.x3dpoint( 0, 0, (map_size+1)*tile_size) )
	endcap[2]:setPosition( xcore.x3dpoint( (map_size+1)*tile_size, 0, 0) )
	endcap[3]:setPosition( xcore.x3dpoint( (map_size+1)*tile_size, 0, (map_size+1)*tile_size) )
	endcap[4]:setPosition( xcore.x3dpoint( 0, 0, 0) )
end

function map:addDoors()
	for x = 1, map_size do
		for y = 1, map_size do
			if self.tiles[ x ][ y ].tile_type == TILE_TYPE_CLEAR then
				if self.tiles[ x ][ y ].text == "Door" then
					-- add door here
				end
			end
		end
	end
end

function map:findRoom()
	-- print("[map] findRoom")

	dir = math.random(1,4)
	randx = math.random(2,map_size-1)
	randy = math.random(2,map_size-1)
	if dir > 2 then
		if self.tiles[ randx ][ randy ].tile_type == TILE_TYPE_CLEAR and self.tiles[ randx-1 ][ randy-1 ].tile_type == TILE_TYPE_CLEAR and
			self.tiles[ randx-1 ][ randy ].tile_type == TILE_TYPE_CLEAR and self.tiles[ randx ][ randy-1 ].tile_type == TILE_TYPE_CLEAR then
			if self.tiles[ randx ][ randy ].text == "Room" and  self.tiles[ randx-1 ][ randy-1 ].text == "Room" and
				self.tiles[ randx-1 ][ randy ].text == "Room" and  self.tiles[ randx ][ randy-1 ].text == "Room" then
				-- print("[map] found room...")
				self.tiles[ randx ][ randy ]:setGraphic( "hay_06.png" )
				self.tiles[ randx ][ randy-1 ]:setGraphic( "hay_02.png" )
				self.tiles[ randx-1 ][ randy ]:setGraphic( "hay_05.png" )
				self.tiles[ randx-1 ][ randy-1 ]:setGraphic( "hay_01.png" )
				self.tiles[ randx ][ randy].text = "Floor"
				self.tiles[ randx ][ randy-1 ].text = "Floor"
				self.tiles[ randx-1 ][ randy ].text = "Floor"
				self.tiles[ randx-1 ][ randy-1 ].text = "Floor"
			end
		end
	end
end

-- removeWalls
function map:removeWalls()
	randx = math.random(2,map_size-1)
	randy = math.random(2,map_size-1)
	if self.tiles[ randx ][ randy ].tile_type == TILE_TYPE_BLOCK then
		-- test to see if there is adjacent clear spots
		if self.tiles[ randx+1 ][ randy ].tile_type == TILE_TYPE_CLEAR and  self.tiles[ randx-1 ][ randy ].tile_type == TILE_TYPE_CLEAR then
			-- if self.tiles[ randx ][ randy+1 ].text ~= "Door" and  self.tiles[ randx ][ randy-1 ].text ~= "Door" then
				-- add a feature to the spot
				self.tiles[ randx ][ randy ]:setType( TILE_TYPE_CLEAR )
				self.tiles[ randx ][ randy ]:setText( "Doorway")
			-- end
		elseif self.tiles[ randx ][ randy+1 ].tile_type == TILE_TYPE_CLEAR and  self.tiles[ randx ][ randy-1 ].tile_type == TILE_TYPE_CLEAR then
			-- if self.tiles[ randx ][ randy+1 ].text ~= "Door" and  self.tiles[ randx ][ randy-1 ].text ~= "Door" then
				-- add a feature to the spot
				self.tiles[ randx ][ randy ]:setType( TILE_TYPE_CLEAR )
				self.tiles[ randx ][ randy ]:setText( "Doorway")
			-- end
		end
	end
end

-- returns a room tile
function map:findRandomStart( e )
	for x = 1, (map_size*map_size)*10 do
		randx = math.random(2,map_size-1)
		randy = math.random(2,map_size-1)
		if self.tiles[ randx ][ randy ].tile_type == TILE_TYPE_CLEAR then
			if self.tiles[ randx ][ randy ].text == "Room" then
				-- make sure start and exit are not too close
				if getDistance( xcore.x2dpoint( randx, randy ), xcore.x2dpoint(e.X, e.Y ) ) > 8 then
					if self.tiles[ randx+1 ][ randy ].tile_type == TILE_TYPE_BLOCK or self.tiles[ randx-1 ][ randy ].tile_type == TILE_TYPE_BLOCK or
						self.tiles[ randx ][ randy-1 ].tile_type == TILE_TYPE_BLOCK or self.tiles[ randx ][ randy-1 ].tile_type == TILE_TYPE_BLOCK then
						-- print "we have a winner"
						return xcore.x2dpoint( randx, randy )
					end
				end
			end
		end
	end
	return nil
end

function map:findStart()
	for x = 1, map_size do
		for y = 1, map_size do
			if self.tiles[ x ][ y ].tile_type == TILE_TYPE_CLEAR then
				if self.tiles[ x ][ y ].text == "Room" then
					-- make sure start and exit are not too close
					if self.tiles[ x+1 ][ y ].tile_type == TILE_TYPE_BLOCK or self.tiles[ x-1 ][ y ].tile_type == TILE_TYPE_BLOCK or
						self.tiles[ x ][ y-1 ].tile_type == TILE_TYPE_BLOCK or self.tiles[ x ][ y-1 ].tile_type == TILE_TYPE_BLOCK then
						-- print "we have a winner"
						return xcore.x2dpoint( x, y )
					end
				end
			end
		end
	end
	return nil
end

-- returns a dead end tile, w/ 3 adjacent walls
function map:getDeadEnd()
	-- for i = 1, map_size * map_size do
	for x = 1, map_size do
		for y = 1, map_size do
			if self.tiles[ x ][ y ].text == "Dead End" then
				-- print("test dead end")
				-- test for 3 walls
				n = 0
				if self.tiles[ x+1 ][ y ].tile_type == TILE_TYPE_BLOCK then n = n+ 1 end
				if self.tiles[ x-1 ][ y ].tile_type == TILE_TYPE_BLOCK then n = n+ 1 end
				if self.tiles[ x ][ y+1 ].tile_type == TILE_TYPE_BLOCK then n = n+ 1 end
				if self.tiles[ x ][ y-1 ].tile_type == TILE_TYPE_BLOCK then n = n+ 1 end
				
				if n >= 3 then
					-- print("found dead end")
					return xcore.x2dpoint( x, y )
				end
			end
		end
	end
	return nil
end

-- returns a dead end tile, w/ 3 adjacent walls
function map:getStairWell()
	-- for i = 1, map_size * map_size do
	for x = map_size, 1, -1 do
		for y = map_size, 1, -1 do
			if self.tiles[ x ][ y ].tile_type == TILE_TYPE_CLEAR and self.tiles[ x ][ y ].text == "Room" then
				-- print("[map] test stair well")
				-- test for 3 walls
				n = 0
				if self.tiles[ x+1 ][ y ].tile_type == TILE_TYPE_BLOCK then n = n+ 1 end
				if self.tiles[ x-1 ][ y ].tile_type == TILE_TYPE_BLOCK then n = n+ 1 end
				if self.tiles[ x ][ y+1 ].tile_type == TILE_TYPE_BLOCK then n = n+ 1 end
				if self.tiles[ x ][ y-1 ].tile_type == TILE_TYPE_BLOCK then n = n+ 1 end
				
				if n >= 2 then
					-- print("[map] found stair well")
					return xcore.x2dpoint( x, y )
				end
			end
		end
	end
	return nil
end

function map:getStart()
	for x = 1, map_size do
		for y = 1, map_size do
			if self.tiles[ x ][ y ].tile_type == 2 then
				return xcore.x2dpoint(x, y)
			end
		end
	end
	return nil
end

function map:stepMaze()
	-- print("[map] stepping maze")
	-- print("[map] map_feature_count : " .. map_feature_count )
	current_location = self:findWall()
	if current_location ~= nil then
		-- print("current_location: " .. current_location.X .. ", " .. current_location.Y)	
		self.tiles[ current_location.X ][ current_location.Y ]:setText( "check" )
		-- add a feature to the spot
		self:makeCorridor( current_location,  math.random(4,9), math.random(1, 4) )
	end
	-- (do this a bunch)
end

function map:stepDungeon()
	-- print("[map] stepping dungeon")
	-- print("[map] map_feature_count : " .. map_feature_count )
	current_location = self:findWall()
	if current_location ~= nil then
		
		-- print("current_location: " .. current_location.X .. ", " .. current_location.Y)	
		self.tiles[ current_location.X ][ current_location.Y ]:setText( "check" )
		-- add a feature to the spot
		room_width = math.random(5,9)
		room_length = math.random(5,9)
		if math.random(1, 100) > 75 then
			-- add a hallway
			self:makeCorridor( current_location, room_length, math.random(1, 4) )
		else
			-- add a square room
			if self:makeRoom( current_location, room_width, room_length, math.random(1, 4) ) == true then
				-- add a doorway
				self.tiles[ current_location.X ][ current_location.Y ]:setText( "Door" )
				self.tiles[ current_location.X ][ current_location.Y ]:setType( TILE_TYPE_CLEAR )
			end
		end
	end
	-- (do this a bunch)
end

function map:saveInventory()
	-- print("[map] saveInventory")
	-- dump everything that currently exists in db first (to avoid dupes or having to track item ids)
	query = "DELETE FROM table_inventory WHERE inventory_storage = 0"
	q = sql:exe( query )
	-- then save everything from memory.
	for i = 1, table.getn( local_user.inventory ) do
		statement = 			 " INSERT INTO table_inventory "
		statement = statement .. " ( inventory_item_id, inventory_user_id, inventory_storage ) "
		statement = statement .. " VALUES ( " .. local_user.inventory[ i ].item_id  .. ", 1, 0 )"
		sql:exe( statement )
	end	
	
	-- don't forget to remove items from memroy, otw they get duped when u load a new map.
	if local_user.inventory ~= nil then
		n = table.getn( local_user.inventory )
		for i=n,1,-1 do
			local_user.inventory[ i ]:destroy()
			table.remove ( local_user.inventory, i )
		end
	end	
end

function map:loadInventory()
	-- print("[map] loadInventory")
	-- load everything from db.
	-- clear inventory?
	-- table.remove( local_user.inventory, listbox_inventory:getSelected()+1 )
	-- local_user.inventory = nil
	
	statement =             " SELECT inventory_item_id, inventory_user_id, inventory_storage "
	statement = statement .." FROM table_inventory "
	statement = statement .." WHERE inventory_user_id = 1 AND inventory_storage = 0"
	-- print( statement )
	local q = sql:exe( statement )

	if q ~= nil then
		-- print ("returned " .. q:getRowCount() .. " rows ")		
		for i = 0, q:getRowCount()-1 do
			local n = i*q:getHeadCount()
			item_count = table.getn( local_user.inventory )
			local_user.inventory[ item_count + 1 ] = item:new()
			local_user.inventory[ item_count + 1 ].item_id = q:getData( n+0 )
		end
	end
	
	for i = 1, table.getn( local_user.inventory ) do
		local_user.inventory[ i ]:loadItem( local_user.inventory[ i ].item_id )
	end
end	

function map:saveParty()
	-- print("[map] saveParty")
	for i = 1, table.getn( self.mobs ) do
		if self.mobs[ i ].owner_id == 1 then
			-- update mob
			-- print("[map] update mobs")
			query = "UPDATE table_characters SET "
			query = query .. " character_level = '" .. self.mobs[ i ]:getLevel()  .. "', "
			query = query .. " character_exp = " .. self.mobs[ i ].exp  .. ", "
			if self.mobs[ i ].wpn_equipped_id ~= 0 then query = query .. " character_wpn = " .. self.mobs[ i ].wpn_equipped_id  .. ", " end
			if self.mobs[ i ].arm_equipped_id ~= 0 then query = query .. " character_arm = " .. self.mobs[ i ].arm_equipped_id  .. ", " end
			if self.mobs[ i ].acc_equipped_id ~= 0 then query = query .. " character_itm = " .. self.mobs[ i ].acc_equipped_id  .. ", " end
			query = query .. " character_hp = " .. self.mobs[ i ].hp .. ", "
			query = query .. " character_sp = " .. self.mobs[ i ].sp .. ", "
			query = query .. " character_state = " .. self.mobs[ i ].state .. ", "
			query = query .. " character_status = " .. self.mobs[ i ].status .. " "
			query = query .. " WHERE character_id = " .. self.mobs[ i ].mob_id

			-- print ( query )
			q = sql:exe( query )
		end
	end
	
end

function map:loadParty()
	-- print("[map] loadParty")
	-- add user's mobs
	--[[
	for i = 1, map_party do
		n = table.getn( self.mobs ) + 1
		self.count = self.count + 1
		self.mobs[ n ] = mob:new()
		self.mobs[ n ].owner_id = 1
		self.mobs[ n ].mob_id = self.count
		self.mobs[ n ]:setClass( groupmembers[ i ] )
		self.mobs[ n ].name      = editbox_select_names[ i ]:getText() -- "User Mob " .. i		
		self.mobs[ n ]:setGrid( -10000, -10000  )
	end
	]]--

	groupmembers = {}
	
	statement =             " SELECT character_id, character_name, character_class, character_level, character_exp, character_wpn, character_arm, character_itm, character_status, character_hp, character_sp, character_state "
	statement = statement .." FROM table_characters "
	statement = statement .." WHERE character_owner_id = 1 "
	-- print( statement )
	local q = sql:exe( statement )

	if q ~= nil then
		-- print ("returned " .. q:getRowCount() .. " rows ")		
		for i = 0, q:getRowCount()-1 do
			local n = i*q:getHeadCount()
			table_creator_roster:addRow( i+1 )

			-- print("\ncharacter_id: " .. q:getData( n+0 ) )
			-- print("character_name: " .. q:getData( n+1 ) )
			-- print("character_class: " .. q:getData( n+2 ) )
			-- print("character_level: " .. q:getData( n+3 ) )
			-- print("character_exp: " .. q:getData( n+4 ) )
			-- print("character_wpn: " .. q:getData( n+5 ) )
			-- print("character_arm: " .. q:getData( n+6) )
			-- print("character_itm: " .. q:getData( n+7 ) )
			-- print("character_status: " .. q:getData( n+8 ) )
			-- print("character_hp: " .. q:getData( n+9 ) )
			-- print("character_sp: " .. q:getData( n+10 ) )
			-- print("character_state: " .. q:getData( n+11 ) )
			
			groupmembers[i] = {}
			groupmembers[i]["id"] = tonumber( q:getData( n+0 ) )
			groupmembers[i]["name"] = q:getData( n+1 )
			groupmembers[i]["class"] =  tonumber( q:getData( n+2 ) )
			if tonumber( q:getData( n+3 ) ) ~= nil then groupmembers[i]["level"] = tonumber( q:getData( n+3 ) ) else groupmembers[i]["level"] = 1 end
			if tonumber( q:getData( n+4 ) ) ~= nil then groupmembers[i]["exp"] = tonumber( q:getData( n+4 ) ) else groupmembers[i]["exp"] = 0 end

			if tonumber( q:getData( n+5 ) ) ~= nil then groupmembers[i]["wep"] = tonumber( q:getData( n+5 ) ) else groupmembers[i]["wep"] = 0 end
			if tonumber( q:getData( n+6 ) ) ~= nil then groupmembers[i]["arm"] = tonumber( q:getData( n+6 ) ) else groupmembers[i]["arm"] = 0 end
			if tonumber( q:getData( n+7 ) ) ~= nil then groupmembers[i]["itm"] = tonumber( q:getData( n+7 ) ) else groupmembers[i]["itm"] = 0 end
			
			if tonumber( q:getData( n+8 ) ) ~= nil then groupmembers[i]["status"] = tonumber( q:getData( n+8 ) ) else groupmembers[i]["status"] = 0 end
			if tonumber( q:getData( n+9 ) ) ~= nil then groupmembers[i]["hp"] = tonumber( q:getData( n+9 ) ) else groupmembers[i]["hp"] = 1 end
			if tonumber( q:getData( n+10 ) ) ~= nil then groupmembers[i]["sp"] = tonumber( q:getData( n+10 ) ) else groupmembers[i]["sp"] = 1 end
			if tonumber( q:getData( n+11 ) ) ~= nil then groupmembers[i]["state"] = tonumber( q:getData( n+11 ) ) else groupmembers[i]["state"] = STATE_RESERVE end
		end
	end
	sql:removeResult( q )
	self:loadInventory()
end

function map:spawnUser()
	-- print("[map] spawnUser")
	self:loadParty()
	-- add user's mobs
	-- map_party = table.getn( groupmembers )
	for i = 0, table.getn( groupmembers ) do
		n = table.getn( self.mobs ) + 1
		self.count = self.count + 1
		self.mobs[ n ] = mob:new()
		self.mobs[ n ].owner_id = 1
		self.mobs[ n ].mob_id = groupmembers[ i ]["id"]
		self.mobs[ n ]:setLevel( groupmembers[ i ]["level"] )		
		self.mobs[ n ].exp      = groupmembers[ i ]["exp"]
		self.mobs[ n ]:setClass( groupmembers[ i ]["class"] )
		self.mobs[ n ].name      = groupmembers[ i ]["name"]
		self.mobs[ n ].status      = groupmembers[ i ]["status"]
		self.mobs[ n ].state      = groupmembers[ i ]["state"]
		self.mobs[ n ].hp      = groupmembers[ i ]["hp"]
		self.mobs[ n ].sp      = groupmembers[ i ]["sp"]
		self.mobs[ n ]:setGrid( -10000, -10000  )
		
		if groupmembers[i]["wep"] ~= 0 then self.mobs[ n ]:equipWeapon( groupmembers[i]["wep"] ) end
		if groupmembers[i]["arm"] ~= 0 then self.mobs[ n ]:equipArmor( groupmembers[i]["arm"] ) end
		if groupmembers[i]["itm"] ~= 0 then self.mobs[ n ]:equipAccessory( groupmembers[i]["itm"] ) end
	end
end

function map:loadDropTable()
	self.drops = {}
	statement = "SELECT item_id FROM table_items WHERE item_rarity <= " .. map_sequence
	local q = sql:exe( statement )
	if q ~= nil then
		for i = 1, q:getRowCount() do
			self.drops[ i ] = q:getData( i-1 )
		end
		self.max_item_types = q:getRowCount()
		sql:removeResult( q )
	end
end

-- todo, fix me...
function map:loadEncounterTable()
	-- print("[map] loadEncounterTable")
	self.encounters = {}
	
	self.max_encounter_types = 0
	statement = "SELECT group_id FROM table_groups WHERE group_level_min <= " .. dungeon_level .. " AND group_level_max >= " .. dungeon_level .. " AND group_sequence = " .. map_sequence
	local q = sql:exe( statement )
	if q ~= nil then
		for i = 1, q:getRowCount() do
			self.encounters[ i ] = q:getData( i-1 )
		end
		self.max_encounter_types = q:getRowCount()
		sql:removeResult( q )
	else
		print("[map] error : no encounter data for level " .. dungeon_level )
	end
	
	-- print( "[map] max_encounter_types : " .. self.max_encounter_types )
	if self.max_encounter_types == 0 then return nil end
	rng = math.random (1, self.max_encounter_types )
	-- print("[map] system has selected encounter id : " .. self.encounters[ rng ] )
	
	statement =             " SELECT encounter_mob_id, encounter_mobs_min, encounter_mobs_max, encounter_chance "
	statement = statement .." FROM table_encounters "
	statement = statement .." WHERE encounter_group_id = " .. self.encounters[ rng ]
	local q = sql:exe( statement )
	if q ~= nil then
		-- print ("returned " .. q:getRowCount() .. " rows ")
		for i = 0, q:getRowCount()-1 do
			local n = i*q:getHeadCount()

			-- if q:getData( n+0 ) ~= nil then print("encounter_mob_id" ..  q:getData( n+0 ) ) end
			-- if q:getData( n+1 ) ~= nil then print("encounter_mobs_min" .. q:getData( n+1 ) ) end	
			-- if q:getData( n+2 ) ~= nil then print("encounter_mobs_max" .. q:getData( n+2 ) ) end
			-- if q:getData( n+3 ) ~= nil then print("encounter_chance" .. q:getData( n+3 ) ) end
			
			local _id = tonumber( q:getData( n+0 ) )
			local _min = tonumber( q:getData( n+1 ) )
			local _max = tonumber( q:getData( n+2 ) )
			local _chance = tonumber( q:getData( n+3 ) )
			if math.random (1, 100 ) <= _chance then
				-- spawn it
				ct = math.random (_min, _max )
				
				for i = 1, ct do
					spawnpoint = self:findSpawn()
					if spawnpoint ~= nil then
						n = table.getn( self.mobs ) + 1
						self.count = self.count + 1
						self.mobs[ n ] = mob:new()
						self.mobs[ n ].owner_id = 2
						self.mobs[ n ].mob_id = n
						self.mobs[ n ].status = STATUS_NORMAL
						self.mobs[ n ]:setState( STATE_READY )
						
						-- find the correct level range this mob should be
						level_range = map_level_max - map_level_min
						levels_per_stage = level_range / map_depth
						if level_range > 0 then
							step_map_level = dungeon_level * levels_per_stage
							self.mobs[ n ]:setLevel( math.random( map_level_min, math.floor( map_level_min+step_map_level ) ) )
						else
							self.mobs[ n ]:setLevel( math.random( map_level_min, map_level_max ) )
						end

						self.mobs[ n ]:setClass( _id )
						-- self.mobs[ n ].name      = "Enemy Mob " .. i		
						self.mobs[ n ]:setGrid( spawnpoint.X, spawnpoint.Y  )
					end
				end
			end
		end
		sql:removeResult( q )		
	end
end

function map:populate()
	-- print("[map] populate")
	self:loadEncounterTable()
	self:loadDropTable()
	
	for i = 1, map_nodes do
		spawnpoint = self:findSpawn()
		if spawnpoint ~= nil then
			self:dropLoot( spawnpoint.X, spawnpoint.Y  )
		end
	end
end

-- drop random loot item
function map:dropLoot( x, y)
	-- print( "[state_game] dropLoot" )
	n = table.getn( self.nodes ) + 1
	self.count = self.count + 1
	self.nodes[ n ] = node:new()
	self.nodes[ n ]:loadGraphic()
	
	if math.random(1, 100) > 50 then
		self.nodes[ n ]:addItem( self.drops[ math.random (1, self.max_item_types) ]  )
	else
		self.nodes[ n ]:addGold( rollDice( map_sequence+1 ) )
	end
	
	self.nodes[ n ]:setGrid( x, y  )
end

-- wtf am i doing here?
function map:update()
	-- print("update")
end

function map:findWall()
	-- print("findWall")
	for i = 1, map_size * map_size do
		point = xcore.x2dpoint( math.random(2,map_size-1), math.random(2,map_size-1))
		if self.tiles[ point.X ][ point.Y ].tile_type == TILE_TYPE_BLOCK then
			-- found a wall... check to see if it next to a clear spot
			if self.tiles[ point.X-1 ][ point.Y ].tile_type == TILE_TYPE_CLEAR or
				self.tiles[ point.X+1 ][ point.Y ].tile_type == TILE_TYPE_CLEAR or 
				 self.tiles[ point.X ][ point.Y-1 ].tile_type == TILE_TYPE_CLEAR or
				  self.tiles[ point.X ][ point.Y+1 ].tile_type == TILE_TYPE_CLEAR then			
				return point
			end
		end
	end
	return nil
end

function map:findFloor()
	-- return true if this tile is clear and also adjacent to nothing but clear
	-- print("findFloor")
	for i = 1, map_size * map_size do
		point = xcore.x2dpoint( math.random(2,map_size-1), math.random(2,map_size-1))
		if self.tiles[ point.X ][ point.Y ].tile_type == TILE_TYPE_CLEAR then
			-- print("found empty spot...")
			if self.tiles[ point.X-1 ][ point.Y ].tile_type == TILE_TYPE_CLEAR or
				self.tiles[ point.X+1 ][ point.Y ].tile_type == TILE_TYPE_CLEAR or 
				 self.tiles[ point.X ][ point.Y-1 ].tile_type == TILE_TYPE_CLEAR or
				  self.tiles[ point.X ][ point.Y+1 ].tile_type == TILE_TYPE_CLEAR then
				return point	
			end
		end
	end
	return nil
end

function map:findSpawn()
	-- return true if this tile is adjacent to nothing but clear
	for i = 1, map_size * map_size do
		point = xcore.x2dpoint( math.random(2,map_size-1), math.random(2,map_size-1))
		if self.tiles[ point.X ][ point.Y ].tile_type == TILE_TYPE_CLEAR then
			is_clear = 1
			for i = 1, table.getn( self.mobs ) do
				if self.mobs[i].grid.X == point.X and self.mobs[i].grid.Y == point.Y then
					is_clear = 0
				end
			end
			for i = 1, table.getn( self.nodes ) do
				if self.nodes[i].grid.X == point.X and self.nodes[i].grid.Y == point.Y then
					is_clear = 0
				end
			end
			
			if is_clear == 1 then return point end
		end
	end
	return nil	
end

function map:makeCorridor( pos, len, direction)
	map_feature_count = map_feature_count+1
	-- print("[map] makeCorridor")
	-- print("[map] direction : " .. direction)
	-- print("[map] length : " .. len)	
	local x = pos.X
	local y = pos.Y
	
	map_feature_type = "Hall"
	
	len = math.random(3, len)
	-- the tile type it's going to be filled with
	-- floor = TILE_TYPE_CLEAR
	-- wall = TILE_TYPE_BLOCKED
	-- print("[map] pos : " .. x.. ", " .. y )	
	
	-- choose the way it's pointing at
	dir = 0
	if direction > 0 and direction < 4 then dir = direction end
	-- print("[map] dir : " .. dir)
	-- print("[map] len : " .. len)
	if dir == 0 then
		-- north
		-- Check if there's enough space left for it
		xtemp = x
		if xtemp+1 < 1 or xtemp+1 > map_size then return false end
		if xtemp-1 < 1 or xtemp-1 > map_size then return false end
		
		for ytemp = y, y+ylen do
			if ytemp < 1 or ytemp > map_size then return false end
			if xtemp == pos.X and ytemp == pos.Y then
				-- print("[map] start square")
			else
				-- print("[map] check space")
				if self:getCell(xtemp, ytemp) ~= TILE_TYPE_UNUSED then return false end
			end
		end
		-- print("[map] building north hall")
		-- we're still here, build
		for ytemp = y, y+ylen do
			-- print("[map] hall : " .. xtemp .. "," .. ytemp)
			self:setCell(xtemp, ytemp, TILE_TYPE_CLEAR )
			self:setCell(xtemp-1, ytemp, TILE_TYPE_BLOCK )
			self:setCell(xtemp+1, ytemp, TILE_TYPE_BLOCK )
			if ytemp == y+ylen then
				self:setCell(xtemp, ytemp, TILE_TYPE_BLOCK )
			elseif ytemp == y+ylen-1 then
				self.tiles[ math.floor(xtemp) ][ math.floor(ytemp) ]:setText( "Dead End" )
			end
		end
	elseif dir == 1 then
		-- south
		-- Check if there's enough space left for it
		xtemp = x
		if xtemp+1 < 1 or xtemp+1 > map_size then return false end
		if xtemp-1 < 1 or xtemp-1 > map_size then return false end
		
		for ytemp = y-ylen, y do
			if ytemp < 1 or ytemp > map_size then return false end
			if xtemp == pos.X and ytemp == pos.Y then
				-- print("[map] start square")
			else
				-- print("[map] check space")
				if self:getCell(xtemp, ytemp) ~= TILE_TYPE_UNUSED then return false end
			end
		end
		-- print("[map] building south hall")
		-- we're still here, build
		for ytemp = y-ylen, y do
			-- print("[map] hall : " .. xtemp .. "," .. ytemp)
			self:setCell(xtemp, ytemp, TILE_TYPE_CLEAR )
			self:setCell(xtemp-1, ytemp, TILE_TYPE_BLOCK )
			self:setCell(xtemp+1, ytemp, TILE_TYPE_BLOCK )
			if ytemp == y-ylen then
				self:setCell(xtemp, ytemp, TILE_TYPE_BLOCK )
			elseif ytemp == y-ylen+1 then
				self.tiles[ math.floor(xtemp) ][ math.floor(ytemp) ]:setText( "Dead End" )
			end	
		end
	elseif dir == 2 then
		-- east
		-- Check if there's enough space left for it
		ytemp = y
		if ytemp+1 < 1 or ytemp+1 > map_size then return false end
		if ytemp-1 < 1 or ytemp-1 > map_size then return false end
		
		for xtemp = x-xlen, x do
			if xtemp < 1 or xtemp > map_size then return false end
			if xtemp == pos.X and ytemp == pos.Y then
				-- print("[map] start square")
			else
				-- print("[map] check space")
				if self:getCell(xtemp, ytemp) ~= TILE_TYPE_UNUSED then return false end
			end
		end
		-- print("[map] building east hall")
		for xtemp = x-xlen, x do
			-- print("[map] hall : " .. xtemp .. "," .. ytemp)
			self:setCell(xtemp, ytemp, TILE_TYPE_CLEAR )
			self:setCell(xtemp, ytemp+1, TILE_TYPE_BLOCK )
			self:setCell(xtemp, ytemp-1, TILE_TYPE_BLOCK )
			if xtemp == x-xlen then
				self:setCell(xtemp, ytemp, TILE_TYPE_BLOCK )
			elseif xtemp == x-xlen+1 then
				self.tiles[ math.floor(xtemp) ][ math.floor(ytemp) ]:setText( "Dead End" )				
			end
		end
	elseif dir == 3 then
		-- west
		-- Check if there's enough space left for it
		ytemp = y
		if ytemp+1 < 1 or ytemp+1 > map_size then return false end
		if ytemp-1 < 1 or ytemp-1 > map_size then return false end
			
		for xtemp = x, x+xlen do
			if xtemp < 1 or xtemp > map_size then return false end

			if xtemp == pos.X and ytemp == pos.Y then
				-- print("[map] start square")
			else
				-- print("[map] check space")
				if self:getCell(xtemp, ytemp) ~= TILE_TYPE_UNUSED then return false end
			end
		end
		-- print("[map] building west hall")
		for xtemp = x, x+xlen do
			-- print("[map] hall : " .. xtemp .. "," .. ytemp)
			self:setCell(xtemp, ytemp, TILE_TYPE_CLEAR )
			self:setCell(xtemp, ytemp+1, TILE_TYPE_BLOCK )
			self:setCell(xtemp, ytemp-1, TILE_TYPE_BLOCK )			
			if xtemp == x+xlen then
				self:setCell(xtemp, ytemp, TILE_TYPE_BLOCK )
			elseif xtemp == x+xlen-1 then
				self.tiles[ math.floor(xtemp) ][ math.floor(ytemp) ]:setText( "Dead End" )				
			end			
		end
	end
	-- self.tiles[ x ][ y ]:setText( "Door")
end

function map:makeRoom( pos, xlength, ylength, direction)
	-- print("[map] makeRoom")
	map_feature_count = map_feature_count+1
	-- print("[map] direction : " .. direction)
	local x = pos.X
	local y = pos.Y

	map_feature_type = "Room"
	
	xlen = xlength
	ylen = ylength
	-- the tile type it's going to be filled with
	-- floor = TILE_TYPE_CLEAR
	-- wall = TILE_TYPE_BLOCKED
	
	-- choose the way it's pointing at
	dir = 0
	if direction > 0 and direction < 4 then dir = direction end

	if dir == 0 then
		-- north
		-- Check if there's enough space left for it
		for ytemp = y, (y-ylen), -1 do
			if ytemp < 1 or ytemp > map_size then return false end
			for xtemp = (x-xlen/2), (x+(xlen)/2) do
				if xtemp < 1 or xtemp > map_size then return false end
				tmp = self:getCell(xtemp, ytemp)
				if tmp ~= TILE_TYPE_UNUSED and tmp ~= TILE_TYPE_BLOCK then return false end
			end
		end

		-- print("[map] building north room")
		for ytemp = y, (y-ylen), -1 do
			for xtemp = (x-xlen/2), (x+(xlen)/2) do		
				if xtemp == (x-xlen/2) then
					self:setCell(xtemp, ytemp, TILE_TYPE_BLOCK)
				elseif xtemp == (x+(xlen)/2) then
					self:setCell(xtemp, ytemp, TILE_TYPE_BLOCK)
				elseif ytemp == y then
					self:setCell(xtemp, ytemp, TILE_TYPE_BLOCK)
				elseif ytemp == (y-ylen) then
					self:setCell(xtemp, ytemp, TILE_TYPE_BLOCK)
				else
					self:setCell(xtemp, ytemp, TILE_TYPE_CLEAR)
				end
			end
		end
	elseif dir == 1 then
	-- east
		-- print("[map] check space")
		for ytemp = (y-ylen/2), (y+(ylen)/2) do
			if ytemp < 1 or ytemp > map_size then return false end
			for xtemp = x, (x+xlen) do
				if xtemp < 1 or xtemp > map_size then return false end
				tmp = self:getCell(xtemp, ytemp)
				if tmp ~= TILE_TYPE_UNUSED and tmp ~= TILE_TYPE_BLOCK then return false end
			end
		end
		-- print("[map] building east room")
		for ytemp = (y-ylen/2), (y+(ylen+1)/2) do
			for xtemp = x, (x+xlen) do
				if xtemp == x then
					self:setCell(xtemp, ytemp, TILE_TYPE_BLOCK)
				elseif xtemp == (x+xlen) then
					self:setCell(xtemp, ytemp, TILE_TYPE_BLOCK)
				elseif ytemp == (y-ylen/2) then
					self:setCell(xtemp, ytemp, TILE_TYPE_BLOCK)
				elseif ytemp == (y+(ylen)/2) then
					self:setCell(xtemp, ytemp, TILE_TYPE_BLOCK)
				else
					self:setCell(xtemp, ytemp, TILE_TYPE_CLEAR)
				end
			end
		end

	elseif dir == 2 then
	-- south
		ytemp = y
		-- print("ytemp:" .. ytemp )
		-- print("y-ylen:" .. y-ylen )		
		for ytemp = y, (y+ylen) do
			if ytemp < 1 or ytemp > map_size then return false end
			for xtemp = (x-xlen/2), (x+(xlen)/2) do
				if xtemp < 1 or xtemp > map_size then return false end
				tmp = self:getCell(xtemp, ytemp)
				if tmp ~= TILE_TYPE_UNUSED and tmp ~= TILE_TYPE_BLOCK then return false end
			end
		end

		for ytemp = y, (y+ylen) do
			for xtemp = (x-xlen/2), (x+(xlen+1)/2) do
				if xtemp == (x-xlen/2) then
					self:setCell(xtemp, ytemp, TILE_TYPE_BLOCK)
				elseif xtemp == (x+(xlen)/2) then
					self:setCell(xtemp, ytemp, TILE_TYPE_BLOCK)
				elseif ytemp == y then
					self:setCell(xtemp, ytemp, TILE_TYPE_BLOCK)
				elseif ytemp == (y+ylen) then
					self:setCell(xtemp, ytemp, TILE_TYPE_BLOCK)
				else
					self:setCell(xtemp, ytemp, TILE_TYPE_CLEAR)
				end
			end
		end

	elseif dir == 3 then
		for ytemp = (y-ylen/2), (y+(ylen+1)/2) do
			if ytemp < 1 or ytemp > map_size then return false end
			for xtemp = x, (x-xlen), -1 do
				if xtemp < 1 or xtemp > map_size then return false end
				tmp = self:getCell(xtemp, ytemp)
				if tmp ~= TILE_TYPE_UNUSED and tmp ~= TILE_TYPE_BLOCK then return false end
			end
		end

		for ytemp = (y-ylen/2), (y+(ylen+1)/2) do
			for xtemp = x, (x-xlen), -1 do
				if xtemp == x then
					self:setCell(xtemp, ytemp, TILE_TYPE_BLOCK)
				elseif xtemp == (x-xlen) then
					self:setCell(xtemp, ytemp, TILE_TYPE_BLOCK)
				elseif ytemp == (y-ylen/2) then
					self:setCell(xtemp, ytemp, TILE_TYPE_BLOCK)
				elseif ytemp == (y+(ylen)/2) then
					self:setCell(xtemp, ytemp, TILE_TYPE_BLOCK)
				else
					self:setCell(xtemp, ytemp, TILE_TYPE_CLEAR)
				end
			end
		end
	end

	self.tiles[ x ][ y ]:setText( "Door")
	-- yay, all done
	return true
end

function map:getCell( x, y )
	-- print("[map] getCell :" .. x .. "," .. y)
	return self.tiles[ math.floor(x) ][ math.floor(y) ].tile_type
end

function map:setCell( x, y, t)
	-- print("[map] setCell")
	self.tiles[ math.floor(x) ][ math.floor(y) ]:setType( t )
	self.tiles[ math.floor(x) ][ math.floor(y) ]:setText( map_feature_type )
end

function map:clear()
	-- print("[map] clear")
	-- destroy map tiles
	if self.tiles ~= nil then
		tile_count = table.getn( self.tiles )
		for x=tile_count,1,-1 do
			for y=tile_count,1,-1 do
				self.tiles[ x ][ y ]:destroy()
				table.remove ( self.tiles[ x ], y )
			end
			table.remove ( self.tiles, x )
		end
		self.tiles = nil
	end
	
	if self.mobs ~= nil then
		-- destroy enemy mobs
		mob_count = table.getn( self.mobs )
		for i=mob_count,1,-1 do
			-- if self.mobs[i].owner_id ~= 1 then
				self.mobs[i]:destroy()
				table.remove ( self.mobs, i )
			-- else
			-- 	self.mobs[i]:setGrid(-10000, -10000)
			-- end
		end
	end
	
	if self.nodes ~= nil then
		-- destroy nodes
		node_count = table.getn( self.nodes )
		for i=node_count,1,-1 do
			self.nodes[i]:destroy()
			table.remove ( self.nodes, i )
		end
	end
end

function map:resetMobs()
	for n = 1, table.getn( self.mobs ) do
		-- self.mobs[n]:reset()
		if self.mobs[n].owner_id == 1 then
			if self.mobs[n].state ~= STATE_DOWN then
				self.mobs[n]:setState( STATE_RESERVE )
			end
		end		
	end
end

function map:load()
end

function map:save()
end

function map:highlightMovement( i )
	for x = 1, map_size do
		for y = 1, map_size do
			-- print("[map] distance: " .. getDistance ( xcore.x3dpoint(x, y, 0), self.mobs[ i ].grid ) )
			-- print("[map] findpath: " ..  findPath( self.mobs[ i ].grid, xcore.x2dpoint(x, y ) ) )
			if getDistance( self.mobs[ i ].grid, xcore.x2dpoint(x, y ) ) <= self.mobs[ i ]:getTotalMove() then
				dist = findPath ( self.mobs[ i ].grid, xcore.x2dpoint(x, y ) )
				if dist > self.mobs[ i ]:getTotalMove() or dist < 1 then
					self.tiles[ x ][ y ].selector:setVisible( false )
					self.tiles[ x ][ y ].valid = 0				
				else
					if self.tiles[ x ][ y ].tile_type ~= TILE_TYPE_BLOCK then
						-- test to see if a unit or door is here
						is_obstructed = 0
						if getMobIndexAtGrid( xcore.x2dpoint(x, y ) ) ~= nil then is_obstructed = 1 end
						if is_obstructed == 0 then
							self.tiles[ x ][ y ].selector:setVisible( true )
							self.tiles[ x ][ y ].valid = 1
						else
							self.tiles[ x ][ y ].selector:setVisible( false )
							self.tiles[ x ][ y ].valid = 0
						end
					else
						self.tiles[ x ][ y ].selector:setVisible( false )
						self.tiles[ x ][ y ].valid = 0
					end
				end
			end
		end
	end
end

function map:highlightEnemies( mob_index, special_index )
	-- print("[map] highlightEnemies")
	for x = 1, map_size do
		for y = 1, map_size do
			if getDistance ( xcore.x3dpoint(x, y, 0), self.mobs[ mob_index ].grid ) > self.mobs[ mob_index ].special_abilities[ special_index ].stat_range then
				self.tiles[ x ][ y ].selector:setVisible( false )
				self.tiles[ x ][ y ].valid = 0
			else
				-- test to see if an enemy is here
				is_enemy = 0
				for n = 1, table.getn( self.mobs ) do
					if self.mobs[ n ].grid.X == x and  self.mobs[ n ].grid.Y == y then
						if self.mobs[ n ].owner_id ~= 1 then -- todo: user_id?
						
							if lineofsight( self.mobs[ mob_index ].grid.X, self.mobs[ mob_index ].grid.Y, x, y ) == true then
								is_enemy = 1
							end
						end
					end
				end
				if is_enemy == 1 then
					self.tiles[ x ][ y ].selector:setVisible( true )
					self.tiles[ x ][ y ].valid = 1
				else
					self.tiles[ x ][ y ].selector:setVisible( false )
					self.tiles[ x ][ y ].valid = 0
				end
			end
		end
	end
end

function map:highlightAllies( mob_index, special_index )
	-- print("[map] highlightAllies")
	for x = 1, map_size do
		for y = 1, map_size do
			if getDistance ( xcore.x3dpoint(x, y, 0), self.mobs[ mob_index ].grid ) > self.mobs[ mob_index ].special_abilities[ special_index ].stat_range then
				self.tiles[ x ][ y ].selector:setVisible( false )
				self.tiles[ x ][ y ].valid = 0
			else
				-- test to see if an enemy is here
				is_ally = 0
				for n = 1, table.getn( self.mobs ) do
					if self.mobs[ n ].grid.X == x and  self.mobs[ n ].grid.Y == y then
						if self.mobs[ n ].owner_id == 1 then -- todo: user_id?
							if lineofsight( self.mobs[ mob_index ].grid.X, self.mobs[ mob_index ].grid.Y, x, y ) == true then
								is_ally = 1
							end

						end
					end
				end
				if is_ally == 1 then
					self.tiles[ x ][ y ].selector:setVisible( true )
					self.tiles[ x ][ y ].valid = 1
				else
					self.tiles[ x ][ y ].selector:setVisible( false )
					self.tiles[ x ][ y ].valid = 0
				end
			end
		end
	end
end

function map:highlightAttack( mob_index )
	for x = 1, map_size do
		for y = 1, map_size do
			if getDistance ( xcore.x3dpoint(x, y, 0), self.mobs[ mob_index ].grid ) > self.mobs[ mob_index ]:getTotalRange() then
				self.tiles[ x ][ y ].selector:setVisible( false )
				self.tiles[ x ][ y ].valid = 0
			else
				-- test to see if an enemy is here
				is_enemy = 0
				for n = 1, table.getn( self.mobs ) do
					if self.mobs[ n ].grid.X == x and  self.mobs[ n ].grid.Y == y then
						if self.mobs[ n ].owner_id ~= 1 then -- todo: user_id?
							if lineofsight( self.mobs[ mob_index ].grid.X, self.mobs[ mob_index ].grid.Y, x, y ) == true then
								is_enemy = 1
							end
						end
					end
				end
				if is_enemy == 1 then
					self.tiles[ x ][ y ].selector:setVisible( true )
					self.tiles[ x ][ y ].valid = 1
				else
					self.tiles[ x ][ y ].selector:setVisible( false )
					self.tiles[ x ][ y ].valid = 0
				end
			end
		end
	end
end

function map:clearHighlights()
	for x = 1, map_size do
		for y = 1, map_size do
			self.tiles[ x ][ y ].selector:setVisible(false)	
		end
	end
end

function map:clearLabels()
	for x = 1, map_size do
		for y = 1, map_size do
			self.tiles[ x ][ y ].label:setText("")
		end
	end
end

function map:drawPath()
	-- print("[map]drawPath")
	for i = table.getn( pathList ), 1, -1 do
		pos = getTilePosByID( pathList[ i ].tileID )
		self.tiles[ pos.X ][ pos.Y ].label:setText( table.getn( pathList )-i ) -- i .. " - " .. pathList[ i ].tileID
	end
end

function getDistance( p1, p2 )
	dx = p1.X - p2.X
	dy = p1.Y - p2.Y
	return math.sqrt(dx*dx + dy*dy)
end

function getMobByMobID( i ) -- int
	for n = 1, table.getn( game_map.mobs ) do
		if game_map.mobs[ n ].mob_id == i then	
			return n
		end
	end
	return nil
end

function getMobIndexAtGrid( p ) -- xcore.x2dpoint
	for i = 1, table.getn( game_map.mobs ) do
		if game_map.mobs[ i ].state ~= STATE_DOWN then
			if game_map.mobs[ i ].grid.X == p.X and game_map.mobs[ i ].grid.Y == p.Y then
				return i
			end
		end
	end
	return nil
end

function getNodeIndexAtGrid( p ) -- xcore.x2dpoint
	-- print("[map] getNodeIndexAtGrid - " .. table.getn( game_map.nodes ) )
	for i = 1, table.getn( game_map.nodes ) do
		-- print ( "[map] test:" .. game_map.nodes[ i ].grid.X .. ", " .. game_map.nodes[ i ].grid.Y )
		if game_map.nodes[ i ].grid.X == p.X and game_map.nodes[ i ].grid.Y == p.Y then
			return i
		end
	end
	return nil
end

function getMapTileFromPoint( p ) -- xcore.x2dpoint, pixel co-ords
	-- print "[map] getMapTileFromPoint"
	-- the tile the cursor is on.
	if p == nil then print "nil value" return nil end
	-- print ( "[map] in:" .. p.X .. ", " .. p.Y )
	-- print ( "[map] tile_size:" .. tile_size )
	
	local x = ( p.X + (tile_size/2)) / tile_size
	x = math.floor( x )

	local y = (p.Y+ (tile_size/2)) / tile_size 
	y = math.floor( y )

	-- print ( "[map] out:" .. x .. ", " .. y )
	return xcore.x2dpoint( x, y )
end

--eof