-- Dungeon Tactics
-- Copyright 2012 lukas w. irwin [ lukas.w.irwin@gmail.com ]
-- This program is free software under the terms of the GNU General Public License version 3 or later.
-- node.lua

node = { ID = 0 }

NODE_TYPE_TRAP         = 0
NODE_TYPE_CHEST        = 1
NODE_TYPE_DOOR         = 2

-- creation
function node:new (o)
	-- print "[node] new node"
	o = o or {}
	setmetatable(o, self)
	self.__index = self

	self.node_id   = 0
	self.type      = 0
	self.parent_id = 0
	self.name      = "Sack"
	
	o.grid      = xcore.x2dpoint( 0, 0 )
	o.position  = xcore.x3dpoint( 0, 0, 0 )
	o.location  = xcore.x2dpoint( 0, 0 )

	-- node can be a container for multiple items.
	o.items = {}
	self.offset = 2.5
	self.gold = 0
	
	return o
end

function node:destroy()
	if self.mesh ~= nil then self.mesh:destroy() end
	if self.billboard ~= nil then self.billboard:destroy() end	
end

function node:addItem( i )
	item_count = table.getn( self.items )
	self.items[ item_count + 1 ] = i
end


function node:addGold( i )
	self.gold = i
end

function node:save()
end

function node:loadGraphic()
	-- print "[node] setGraphic"
	self.billboard = sim:addBillboard ( 0, xcore.x2dpoint(4, 4) )
	self.billboard:addTexture( path .. "resources/sprites/sack.png", 0 )
	self.billboard:setMaterialFlag( xcore.XMF_LIGHTING, false )	
	self.billboard:setMaterialType( xcore.XMT_TRANSPARENT_ALPHA_CHANNEL )		
	-- self.billboard:setPosition( xcore.x3dpoint( 10, 16, 10 ) )

	--[[
	self.mesh = sim:addCubeMesh ( 1 )
	self.mesh:setScale( xcore.x3dpoint( 0.5, 0.5, 0.5 ) )

	self.mesh:addTexture( "modules/xhrit_rotd/resources/crate_01.bmp", 0 )
	self.mesh:setMaterialFlag( xcore.XMF_LIGHTING, false )
	]]--
end

function node:getGrid () return self.grid end

function node:setGrid( x, y )
	-- print("[node] setGrid")
	self.grid.X = x
	self.grid.Y = y
	self.position = xcore.x3dpoint( self.grid.X* tile_size, 2, self.grid.Y* tile_size )

	if self.mesh ~= nil then
		self.mesh:setPosition( xcore.x3dpoint( (self.grid.X* tile_size)+self.offset, 2, (self.grid.Y* tile_size)+self.offset ) )
	end
	
	if self.billboard ~= nil then
		self.billboard:setPosition( xcore.x3dpoint( (self.grid.X* tile_size)+self.offset, 2, (self.grid.Y* tile_size)+self.offset ) )
	end
end

function node:setPosition( p )
	self.position = p
	if self.mesh ~= nil then
		self.mesh:setPosition( p)
	end
end

function node:getPosition ()
	return self.position
end

-- end of file