-- Dungeon Tactics
-- Copyright 2012 lukas w. irwin [ lukas.w.irwin@gmail.com ]
-- This program is free software under the terms of the GNU General Public License version 3 or later.
-- state_strategy.lua

state_strategy = state:new()

-- creation
function state_strategy:new (o)
	-- print "[state_strategy] new state_strategy"
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	self.sid = 0

	-- create ui elements
	menu_logistics = logistics:new()   -- add a town menu
	menu_selector = selector:new()     -- quest selector
	menu_creator = creator:new()       -- character creator
	menu_vendor = vendor:new()         -- merchants
	menu_medical = medical:new()       -- heal and rez
	
	map_party = 6 -- todo: bad spot for this.
	
	self:loadGroup()
	self:hide()
	return o
end

function state_strategy:show ()
	-- print "[state_strategy] show"
	menu_logistics:show()

	turn_window:setVisible( true )
	text_game_desc:setVisible( true )
	text_game_desc:setText("")
	
	self:updateParty()
end

function state_strategy:hide ()
	menu_logistics:hide()
	self:hideMenus()

	turn_window:setVisible( false )
	text_game_desc:setText( "" )
end

function state_strategy:loadGroup ()
	groupmembers = {}
end

-- called on state_strategy enter
function state_strategy:enter ()
	-- print "[state_strategy] enter state_strategy"

	-- create (or load) objects and map
	local_user = user:new()	-- user data
	local_user:load()
	image_background:setVisible( true )
		
	-- snd:clear()
	-- if snd:isPlaying( sound_menu_theme ) == false then
	-- 	snd:playSource( sound_menu_theme )
	-- end
	game_camera:reset()

	-- show everything
	self:show()	
end


function state_strategy:updateParty ()
	statement = " SELECT character_id FROM table_characters WHERE character_owner_id = 1 "
	local q = sql:exe( statement )
	if q~= nil then
		local_user.party_count = q:getRowCount()	
		-- print ("returned " .. q:getRowCount() .. " rows ")
	else
		local_user.party_count = 0
	end
	self:update()
end

function state_strategy:update()	
	text_game:setText("Intermission\nParty:" .. local_user.party_count .. "/" .. map_party .. "\nGold:" .. local_user.gold )
end

-- called on state_strategy exit
function state_strategy:exit ()
	-- print "[state_strategy] exit state_strategy"
	-- hide everything
	-- w:setVisible( false )
	self:hide()
	image_background:setVisible( false )

	local_user:save()
	local_user:destroy()
	local_user = nil
end

-- called to advance state_strategy
function state_strategy:input ()
	-- print ( "[state_strategy] input")
	self:update()
	sim:update()

	r = ENUM_STRATEGY
	if sim:isGUI( "button_logistics_selector_pressed" ) then
		-- print("[state_strategy] button_logistics_selector_pressed")	
		self:hideMenus()	
		menu_selector:show()
	end
	if sim:isGUI( "button_logistics_creator_pressed" ) then
		-- print("[state_strategy] button_logistics_creator_pressed")	
		self:hideMenus()
		menu_creator:show()
	end
	if sim:isGUI( "button_logistics_vendor_pressed" ) then
		-- print("[state_strategy] button_logistics_vendor_pressed")
		self:hideMenus()
		menu_vendor:show()
	end
	if sim:isGUI( "button_logistics_medical_pressed" ) then
		-- print("[state_strategy] button_logistics_medical_pressed")
		self:hideMenus()
		menu_medical:show()
	end
	
	if sim:isGUI( "button_logistics_exit_pressed" ) then
		-- print("button_logistics_exit_pressed")
		self:hideMenus()
		game_alert:show()
	end	
	if sim:isGUI( "button_logistics_storage_pressed" ) then	print("button_logistics_storage_pressed stub") end	
	if sim:isGUI( "button_creator_add_pressed" ) then menu_creator:setStep( 2 ) end

	if sim:isGUI( "button_creator_edit_pressed" ) then
		print("[state_strategy] button_creator_edit stub")
		-- menu_creator:setStep( 3 )
	end

	if sim:isGUI( "button_creator_delete_pressed" ) then menu_creator:setStep( 4 ) end

	if sim:isGUI( "button_creator_create_pressed" ) then
		menu_creator:addMob()
		menu_creator:setStep( 1 )
		self:updateParty()
	end	

	if sim:isGUI( "button_creator_cancel_pressed" ) then menu_creator:setStep( 1 ) end	
	if sim:isGUI( "button_creator_next_class_pressed" ) then menu_creator:changeClass( -1 ) end
	if sim:isGUI( "button_creator_back_class_pressed" ) then menu_creator:changeClass( 1 ) end
	if sim:isGUI( "button_creator_yes_pressed" ) then
		menu_creator:deleteMob()
		menu_creator:setStep( 1 )
		self:updateParty()
	end

	if sim:isGUI( "button_creator_no_pressed" ) then menu_creator:setStep( 1 ) end	

	if sim:isGUI( "button_selector_start_pressed" ) then
		-- print("[state_strategy] button_selector_start_pressed")
		if local_user.party_count > 0 then
			if menu_selector:isUnlocked() then 
				self:hideMenus()			
				r = ENUM_GAME
			else
				print("[state_strategy] error: quest locked")
			end
		else
			print("[state_strategy] error: below min party size")
		end
	end
	if sim:isGUI( "button_selector_next_pressed" ) then menu_selector:changeQuest( 1 ) end
	if sim:isGUI( "button_selector_back_pressed" ) then menu_selector:changeQuest( -1 ) end	
	if sim:isGUI( "button_vendor_buy_pressed" ) then menu_vendor:buy() end	
	if sim:isGUI( "button_vendor_sell_pressed" ) then menu_vendor:sell() end	

	if sim:isKey( "key_pressed_exit_state" ) then
		self:hideMenus()
		game_alert:show()		
	end

	if sim:isGUI( "button_alert_accept_pressed" ) then
		print("button_alert_accept_pressed")
		game_alert:hide()
		return ENUM_MENU
	end

	if sim:isGUI( "button_alert_cancel_pressed" ) then
		game_alert:hide()
	end

	if sim:isGUI( "button_medical_heal_pressed" ) then menu_medical:heal() end
	if sim:isGUI( "button_medical_heal_all_pressed" ) then menu_medical:healAll() end



	-- print("update sound...")
	-- snd:setListenerPosition( 0, 0, 0 )
	-- snd:setSourcePosition( sound_menu_theme, 0, 0, 0) -- xcore.x3dpoint( 0, 0, 0 ) 
	-- snd:update()
	-- print("/ update sound")
	return r
end

function state_strategy:hideMenus()
	-- print "[state_strategy] hideMenus"
	-- hideMenus
	menu_medical:hide()
	menu_selector:hide()
	menu_creator:hide()
	menu_vendor:hide()
	game_alert:hide()
end

-- called when changing state_strategy
function state_strategy:transition ( i ) -- transition to state_strategy
	-- print "[state_strategy] transition"
end

-- eof