-- Dungeon Tactics
-- Copyright 2012 lukas w. irwin [ lukas.w.irwin@gmail.com ]
-- This program is free software under the terms of the GNU General Public License version 3 or later.
-- unitlist.lua

unitlist = { ID = 0 }

-- creation
function unitlist:new (o)
	-- print "[unitlist] new unitlist"
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	
	width = 262
	height = 301
	x = screen_width/2 - width - 40
	y = 100

	-- unit window and reserve selection list.
	unitlist_window = sim:addWindow( 300, xcore.x2drect(x, y, x+width, y+height), "Unit List")
	unitlist_window:setTitleBarVisible( false )
	image_background_unitlist = sim:addImage( 0, texture_background_512x512, xcore.x2dpoint(0, 0),
		xcore.x2drect(0,0,64,64), xcore.xcolour(255,255,255,255) )
	image_background_unitlist:setParent( unitlist_window )

	width = 252
	height = 286
	x = 5
	y = 10
	-- unit list table
	table_unit_list = sim:addTable( 1299, true, xcore.x2drect( x+0, y+0, x+width, y+height ) )
	table_unit_list:setParent( unitlist_window )
	table_unit_list:addColumn( "Name       ", 1 )
	table_unit_list:addColumn( "Status  ", 2 )		
	table_unit_list:addColumn( "HP%", 3 )

	sim:bindGUI ( "table_unitlist_select", 1299, xcore.XGUI_TABLE_CHANGED )
	sim:bindGUI ( "table_unitlist_select", 1299, xcore.XGUI_TABLE_SELECTED_AGAIN )

	self:hide()
	return o
end

function unitlist:hide()
	-- print "[unitlist] hide"
	unitlist_window:setVisible( false )
end

function unitlist:show()
	-- print "[unitlist] show"
	self:update()
	unitlist_window:setVisible( true )
	-- print "[unitlist] /show"
end

function unitlist:destroy()
	-- print "[unitlist] destroy"
	unitlist_window:destroy()
end

function unitlist:selectUnit()
	-- print( "[unitlist] select unit" )
	local tableindex = table_unit_list:getSelected()	
	if tableindex == nil then return end
	if tableindex == -1 then return end
	-- print( "[unitlist] Select " .. game_map.mobs[ tableindex+1 ].name )
	
	if game_map.mobs[ tableindex+1 ].state > 0 then
		setSelected( tableindex+1 )
		cam_target = xcore.x2dpoint( game_map.mobs[ tableindex+1 ].grid.X, game_map.mobs[ tableindex+1 ].grid.Y )
	elseif game_map.mobs[ tableindex+1 ].state == STATE_RESERVE then
		-- if reserve then focus start area and bring up base panel	
		-- print("[unitlist] focus start area")
		local pos = game_map:getStart()
		
		if getMobIndexAtGrid( pos ) == nil then
			cam_target = xcore.x2dpoint( pos.X, pos.Y )
			menu_base:deployMobByIndex( tableindex+1 )
			self:hide()
			menu_support:hide()
		else
			-- print("deployment zone is obstructed")
			game_history:addItem( "Deployment zone is obstructed." )
		end
	else
		-- do other stuff.
		-- print("[unitlist] unit not avaliable")
		game_history:addItem( "Unit not avaliable." )
	end
end

function unitlist:update()
	-- print "[unitlist] update"
	table_unit_list:clear()
	table_unit_list:addColumn( "Name       ", 1 )
	table_unit_list:addColumn( "Status  ", 2 )		
	table_unit_list:addColumn( "HP%", 3 )

	for n = 0, table.getn( game_map.mobs )-1 do
		-- print("n:" .. n)
		-- print("game_map.mobs[ n ].name:" .. game_map.mobs[ n+1 ].name)
		table_unit_list:addRow( n )
		if game_map.mobs[ n+1 ].name ~= "none" then
			table_unit_list:setCellText ( n, 0, game_map.mobs[ n+1 ].name )
		else
			table_unit_list:setCellText ( n, 0, game_map.mobs[ n+1 ].class_name )
		end
		
		local damage = game_map.mobs[ n+1 ].hp
		local hp = game_map.mobs[ n+1 ].max_hp * game_map.mobs[ n+1 ].level
		local percent = ( ( hp-damage )/ hp ) * 100
		percent = 100-percent
		table_unit_list:setCellText ( n, 1, getMobState( n+1 ) )
		
		if percent <= 1 then 
			table_unit_list:setCellText ( n, 2, "--" )
		else
			table_unit_list:setCellText ( n, 2, math.floor( percent ) .. "%" )
		end
	end
	-- print "[unitlist] /update"
end

-- end of file