-- xenos particle system editor
-- c2011 Lukas W. Irwin
--

require "lfs"

-- required
function init()
	print "[module] loading particle_editor"

	-- start a simulation
	sim = xcore.xsimulation ( xcore.XDT_OPENGL , 800, 600, 16, false, false, false )
	sim:setWindowCaption( "Xenos Particle System Editor" )

	-- add annotations
	--     | Y
	--     |
	--     |
	--     |_________ X
	--    /
	--   /
	--  / Z
	-- x = red
	-- y = green
	-- z = blue
	line_x = sim:add3dLine ( 0, xcore.x3dpoint( 0, 0, 0 ), xcore.x3dpoint( 10, 0, 0 ) )
	line_x:setColour( xcore.xcolour( 255, 255, 0, 0 ) ) -- red
	
	line_y = sim:add3dLine ( 0, xcore.x3dpoint( 0, 0, 0 ), xcore.x3dpoint( 0, 10, 0 ) )
	line_y:setColour( xcore.xcolour( 255, 0, 255, 0 ) ) -- green
	
	line_z = sim:add3dLine ( 0, xcore.x3dpoint( 0, 0, 0 ), xcore.x3dpoint( 0, 0, 10) )
	line_z:setColour( xcore.xcolour( 255, 0, 0, 255 ) ) -- blue
	
	-- add a static camera
	sim:addCamera ( 0, xcore.CAMERA_STATIC, 100,100,100, 0,0,0)

	x = 20
	y = 20
	-- add a window
	edit_window = sim:addWindow( 100, xcore.x2drect( x, y, x+200, y+320 ), "Edit Window" )

	x = 20
	y = 30	
	text_label_emittertype = sim:addText( 0, "Emitter", x, y, x+50, y+12 )
	text_label_emittertype:setParent( edit_window )
	x = x+40	
	combobox_emittertype = sim:addCombobox( 100, xcore.x2drect(x, y, x+110, y+12 ) )
	combobox_emittertype:setParent( edit_window )
	combobox_emittertype:addItem("Point")
	combobox_emittertype:addItem("Ring")
	combobox_emittertype:addItem("Sphere")
	-- combobox_emittertype:addItem("Box")

	x = 20
	y = y+20	
	text_label_particletype = sim:addText( 0, "Particle", x, y, x+50, y+12 )
	text_label_particletype:setParent( edit_window )
	x = x+40
	combobox_particletype = sim:addCombobox( 100, xcore.x2drect(x, y, x+110, y+12 ) )
	combobox_particletype:setParent( edit_window )
	combobox_particletype:addItem("Fire")
	combobox_particletype:addItem("Smoke")
	combobox_particletype:addItem("Blood")
	combobox_particletype:addItem("Water")

	x = 20
	y = y+20
	-- add numeric inputs for X, Y Z Directions
	text_label_size = sim:addText( 0, "Direction", x, y, x+150, y+12 )
	text_label_size:setParent( edit_window )
	x = x+40
	editbox_dir_x = sim:addEditbox( 20,  xcore.x2drect( x,y,x+30,y+12 ) )
	editbox_dir_x:setParent( edit_window )
	x = x+40
	editbox_dir_y = sim:addEditbox( 20,  xcore.x2drect( x,y,x+30,y+12 ) )
	editbox_dir_y:setParent( edit_window )
	x = x+40
	editbox_dir_z = sim:addEditbox( 20,  xcore.x2drect( x,y,x+30,y+12 ) )
	editbox_dir_z:setParent( edit_window )

	x = 20
	y = y+20
	-- add numeric inputs for min and max size
	text_label_size = sim:addText( 0, "Size", x, y, x+150, y+12 )
	text_label_size:setParent( edit_window )
	x = x+40
	editbox_size_min = sim:addEditbox( 20,  xcore.x2drect( x,y,x+50,y+12 ) )
	editbox_size_min:setParent( edit_window )
	x = x+60
	editbox_size_max = sim:addEditbox( 20,  xcore.x2drect( x,y,x+50,y+12 ) )
	editbox_size_max:setParent( edit_window )

	x = 20
	y = y+20
	-- add numeric inputs for min and max life
	text_label_life = sim:addText( 0, "Lifetime", x, y, x+150, y+12 )
	text_label_life:setParent( edit_window )
	x = x+40
	editbox_life_min = sim:addEditbox( 20,  xcore.x2drect( x,y,x+50,y+12 ) )
	editbox_life_min:setParent( edit_window )
	x = x+60
	editbox_life_max = sim:addEditbox( 20,  xcore.x2drect( x,y,x+50,y+12 ) )
	editbox_life_max:setParent( edit_window )

	x = 20
	y = y+20
	-- add numeric inputs for min and max life
	text_label_pps = sim:addText( 0, "Frequency", x, y, x+150, y+12 )
	text_label_pps:setParent( edit_window )
	x = x+40
	editbox_pps_min = sim:addEditbox( 20,  xcore.x2drect( x,y,x+50,y+12 ) )
	editbox_pps_min:setParent( edit_window )
	x = x+60
	editbox_pps_max = sim:addEditbox( 20,  xcore.x2drect( x,y,x+50,y+12 ) )
	editbox_pps_max:setParent( edit_window )

	x = 20
	y = y+20
	-- add a slider to control angle
	text_label_angle = sim:addText( 0, "Angle", x, y, x+150, y+12 )
	text_label_angle:setParent( edit_window )
	y = y+12
	scrollbar_angle = sim:addScrollbar(100, true, xcore.x2drect(x, y, x+150, y+12 ) )
	scrollbar_angle:setMax( 360 )
	scrollbar_angle:setParent( edit_window )

	x = 20
	y = y+20
	-- add a slider to control radius
	text_label_radius = sim:addText( 0, "Radius", x, y, x+150, y+12 )
	text_label_radius:setParent( edit_window )
	y = y+12
	scrollbar_radius = sim:addScrollbar(100, true, xcore.x2drect(x, y, x+150, y+12 ) )
	scrollbar_radius:setMax( 100 )
	scrollbar_radius:setParent( edit_window )

	x = 20
	y = y+20
	-- add a checkbox and textfield for gravity
	checkbox_grav = sim:addCheckbox( 106, true, "Gravity", x, y, x+100, y+12)
	checkbox_grav:setParent( edit_window )
	x = x+100
	editbox_grav = sim:addEditbox( 20,  xcore.x2drect( x,y,x+50,y+12 ) )
	editbox_grav:setParent( edit_window )

	x = 20
	y = y+20
	-- add a checkbox and textfield for fader
	checkbox_fade = sim:addCheckbox( 106, true, "Fade", x, y, x+100, y+12)
	checkbox_fade:setParent( edit_window )
	x = x+100
	editbox_fade = sim:addEditbox( 20,  xcore.x2drect( x,y,x+50,y+12 ) )
	editbox_fade:setParent( edit_window )

	x = 20
	y = y+20
	-- add a checkbox and textfields for rotation
	checkbox_rotate = sim:addCheckbox( 106, false, "Rotate", x, y, x+100, y+12)
	checkbox_rotate:setParent( edit_window )	
	x = x+50
	editbox_rot_x = sim:addEditbox( 20,  xcore.x2drect( x,y,x+30,y+12 ) )
	editbox_rot_x:setParent( edit_window )
	x = x+35
	editbox_rot_y = sim:addEditbox( 20,  xcore.x2drect( x,y,x+30,y+12 ) )
	editbox_rot_y:setParent( edit_window )
	x = x+35
	editbox_rot_z = sim:addEditbox( 20,  xcore.x2drect( x,y,x+30,y+12 ) )
	editbox_rot_z:setParent( edit_window )

	x = 20
	y = y+25
	-- button to save
	button_save_as = sim:addButton( 202, xcore.x2drect( x, y, x+75, y+12 ), "Save As" )
	button_save_as:setParent( edit_window )
	x = x+75
	-- button to load
	button_load = sim:addButton( 203, xcore.x2drect( x, y, x+75, y+12 ), "Load" )
	button_load:setParent( edit_window )	

	x = 20
	y = y+13
	-- button to apply changes
	button_apply = sim:addButton( 200, xcore.x2drect( x, y, x+75, y+12 ), "Apply" )
	button_apply:setParent( edit_window )
	x = x+75
	-- add button to reset particle system
	button_reset = sim:addButton( 201, xcore.x2drect( x, y, x+75, y+12 ), "Reset" )
	button_reset:setParent( edit_window )

	sim:bindkey ( "quit_key_pressed", xcore.XKEY_ESCAPE )
	sim:bindGUI ( "button_pressed_apply", 200, xcore.XGUI_BUTTON_CLICKED )
	sim:bindGUI ( "button_pressed_reset", 201, xcore.XGUI_BUTTON_CLICKED )
	sim:bindGUI ( "button_pressed_save_as", 202, xcore.XGUI_BUTTON_CLICKED )
	sim:bindGUI ( "button_pressed_load", 203, xcore.XGUI_BUTTON_CLICKED )	

	x = 300
	y = 200
	-- add a window
	save_window = sim:addWindow( 100, xcore.x2drect( x, y, x+200, y+80 ), "Save As" )
	save_window:setVisible( false )	

	x = 20
	y = 30
	-- add text inputs for filename
	text_label_filename = sim:addText( 0, "Filename", x, y, x+100, y+12 )
	text_label_filename:setParent( save_window )
	x = x+45
	editbox_filename = sim:addEditbox( 20,  xcore.x2drect( x,y,x+100,y+12 ) )
	editbox_filename:setParent( save_window )
	
	x = 20
	y = y+25
	-- button to save
	button_save = sim:addButton( 210, xcore.x2drect( x, y, x+75, y+12 ), "Save" )
	button_save:setParent( save_window )
	x = x+75
	-- button to cancel
	button_cancel = sim:addButton( 211, xcore.x2drect( x, y, x+75, y+12 ), "Cancel" )
	button_cancel:setParent( save_window )	

	--
	-- load window
	-- 

	x = 300
	y = 180	
	-- add the window
	load_window = sim:addWindow( 100, xcore.x2drect( x, y, x+200, y+180 ), "Load" )
	load_window:setVisible( false )
	x = 20	
	y = 30
	-- add a listbox for file names
	listbox_filenames = sim:addListbox( 0, true, xcore.x2drect( x,y,x+150,y+100 ) )
	listbox_filenames:setParent( load_window )	
	
	x = 20
	y = y+110
	-- button to load
	button_loadfile = sim:addButton( 212, xcore.x2drect( x, y, x+75, y+12 ), "Load" )
	button_loadfile:setParent( load_window )
	x = x+75
	-- button to cancel
	button_cancel_load = sim:addButton( 211, xcore.x2drect( x, y, x+75, y+12 ), "Cancel" )
	button_cancel_load:setParent( load_window )	


	sim:bindGUI ( "button_pressed_save", 210, xcore.XGUI_BUTTON_CLICKED )
	sim:bindGUI ( "button_pressed_cancel", 211, xcore.XGUI_BUTTON_CLICKED )
	sim:bindGUI ( "button_pressed_loadfile", 212, xcore.XGUI_BUTTON_CLICKED )	
	
	-- reset to sane defaults
	resetDefaultValues()
	createSystem()
	
	
end

-- required
function render()
	-- render simulation tree
	sim:render()
end

-- required
function update()
	if sim:isKey( "quit" ) then sim:destroy() end

	if sim:isGUI( "button_pressed_apply" ) then createSystem() end
	if sim:isGUI( "button_pressed_save_as" ) then showSaveDialog() end
	if sim:isGUI( "button_pressed_save" ) then saveSystem() end	
	if sim:isGUI( "button_pressed_load" ) then showLoadDialog() end	
	if sim:isGUI( "button_pressed_loadfile" ) then loadSystem() end		
	if sim:isGUI( "button_pressed_cancel" ) then cancel() end	
	
	if sim:isGUI( "button_pressed_reset" ) then
		resetDefaultValues()
		createSystem()
	end
	sim:update()
end

function resetDefaultValues()
	particle_minpps = 10
	particle_maxpps = 100
	particle_minlife = 100
	particle_maxlife = 1000
	particle_angle = 0
	particle_minsize = 1
	particle_maxsize = 10
	particle_radius = 20
	particle_thickness = 1
	particle_is_grav = 1
	particle_is_fade = 1
	particle_is_rotate = 0
	particle_grav = 6400
	particle_fade = 1200
	particle_dir_x = 0
	particle_dir_y = 0.1
	particle_dir_z = 0
	particle_rot_x = 0
	particle_rot_y = 0
	particle_rot_z = 0

	particle_emittertype = 0
	particle_particletype = 0 -- stupid name. all of it, i mean... should be field_ or something.

	path = "modules/particle_editor/saves/"
	particle_filename = "particle_system.psf"
	
	setInputs()
end

function setInputs()
	-- print("setting input boxes to varibles")
	editbox_size_min:setText( particle_minsize )
	editbox_size_max:setText( particle_maxsize )
	editbox_pps_min:setText( particle_minpps )
	editbox_pps_max:setText( particle_maxpps )
	editbox_life_min:setText( particle_minlife )	
	editbox_life_max:setText( particle_maxlife )
	editbox_grav:setText( particle_grav )
	editbox_fade:setText( particle_fade )
	
	editbox_dir_x:setText( particle_dir_x )
	editbox_dir_y:setText( particle_dir_y )
	editbox_dir_z:setText( particle_dir_z )
	editbox_rot_x:setText( particle_rot_x )
	editbox_rot_y:setText( particle_rot_y )
	editbox_rot_z:setText( particle_rot_z )
	
	editbox_filename:setText( particle_filename )

	combobox_emittertype:setSelected( particle_emittertype )
	combobox_particletype:setSelected( particle_particletype )
	scrollbar_radius:setPos( particle_radius )
	scrollbar_angle:setPos( particle_angle )
	
	if particle_is_grav == 1 then checkbox_grav:setChecked( true ) else checkbox_grav:setChecked( false ) end
	if particle_is_fade == 1 then checkbox_fade:setChecked( true ) else checkbox_fade:setChecked( false ) end
	if particle_is_rotate == 1 then checkbox_rotate:setChecked( true ) else checkbox_rotate:setChecked( false ) end
	
end

function createSystem()
	-- print("reset particle system")
	if field ~= nil then
		field:destroy()
		field = nil
	end

	-- get data from input boxes
	if tonumber( editbox_size_min:getText() ) ~= nil then particle_minsize = tonumber( editbox_size_min:getText() ) end
	if tonumber( editbox_size_max:getText() ) ~= nil then particle_maxsize = tonumber( editbox_size_max:getText() ) end	
	if tonumber( editbox_life_min:getText() ) ~= nil then particle_minlife = tonumber( editbox_life_min:getText() ) end
	if tonumber( editbox_life_max:getText() ) ~= nil then particle_maxlife = tonumber( editbox_life_max:getText() ) end	
	if tonumber( editbox_pps_min:getText() ) ~= nil then particle_minpps = tonumber( editbox_pps_min:getText() ) end
	if tonumber( editbox_pps_max:getText() ) ~= nil then particle_maxpps = tonumber( editbox_pps_max:getText() ) end
	if tonumber( editbox_grav:getText() ) ~= nil then particle_grav = tonumber( editbox_grav:getText() ) end
	if tonumber( editbox_fade:getText() ) ~= nil then particle_fade = tonumber( editbox_fade:getText() ) end
	if tonumber( editbox_dir_x:getText() ) ~= nil then particle_dir_x = tonumber( editbox_dir_x:getText() ) end	
	if tonumber( editbox_dir_y:getText() ) ~= nil then particle_dir_y = tonumber( editbox_dir_y:getText() ) end	
	if tonumber( editbox_dir_z:getText() ) ~= nil then particle_dir_z = tonumber( editbox_dir_z:getText() ) end	
	if tonumber( editbox_rot_x:getText() ) ~= nil then particle_rot_x = tonumber( editbox_rot_x:getText() ) end	
	if tonumber( editbox_rot_y:getText() ) ~= nil then particle_rot_y = tonumber( editbox_rot_y:getText() ) end	
	if tonumber( editbox_rot_z:getText() ) ~= nil then particle_rot_z = tonumber( editbox_rot_z:getText() ) end		

	particle_angle = scrollbar_angle:getPos()
	particle_radius = scrollbar_radius:getPos()	
	
	particle_emittertype = combobox_emittertype:getSelected()
	particle_particletype = combobox_particletype:getSelected()

	if combobox_particletype:getSelected() == 0 then
		particle_texture = "modules/particle_editor/resources/fire.bmp"
	elseif combobox_particletype:getSelected() == 1 then
		particle_texture = "modules/particle_editor/resources/smoke.bmp"
	elseif combobox_particletype:getSelected() == 2 then
		particle_texture = "modules/particle_editor/resources/blood.bmp"
	elseif combobox_particletype:getSelected() == 3 then
		particle_texture = "modules/particle_editor/resources/water.bmp"
	end

	-- set values
	field = sim:addParticleSystem( 300, particle_texture )
	field:setMaterialFlag( xcore.XMF_LIGHTING, false )
	field:setMaterialFlag( xcore.XMF_ZWRITE_ENABLE, false )
	field:setMaterialType( xcore.XMT_TRANSPARENT_ADD_COLOR )

	if combobox_emittertype:getSelected() == 0 then 
		-- point emitter
		field:createPointEmitter( xcore.x3dpoint( particle_dir_x, particle_dir_y, particle_dir_z), particle_minpps, particle_maxpps,
			xcore.xcolour(255, 255, 255, 255), xcore.xcolour(255, 255, 255, 255),
			particle_minlife, particle_maxlife, particle_angle, particle_minsize, particle_maxsize )
	elseif combobox_emittertype:getSelected() == 1 then
		-- ring emitter
		field:addRingEmitter( xcore.x3dpoint(0, 0, 0), particle_radius, particle_thickness, xcore.x3dpoint(particle_dir_x, particle_dir_y, particle_dir_z),
			particle_minpps, particle_maxpps, xcore.xcolour(255, 255, 255, 255), xcore.xcolour(255, 255, 255, 255),
			particle_minlife, particle_maxlife, particle_angle, particle_minsize, particle_maxsize)
	elseif combobox_emittertype:getSelected() == 2 then
		-- sphere emitter	
		field:addSphereEmitter( xcore.x3dpoint(0, 0, 0), particle_radius, xcore.x3dpoint(particle_dir_x, particle_dir_y, particle_dir_z),
			particle_minpps, particle_maxpps, xcore.xcolour(255, 255, 255, 255), xcore.xcolour(255, 255, 255, 255),
			particle_minlife, particle_maxlife, particle_angle, particle_minsize, particle_maxsize)
	end
	
	if checkbox_grav:isChecked() then
		particle_is_grav = 1
		field:addGravityAffector( xcore.x3dpoint(0,-1, 0 ), particle_grav )
	else
		particle_is_grav = 0
	end
	if checkbox_fade:isChecked() then
		particle_is_fade = 1
		field:addFadeOutAffector( xcore.xcolour(0, 0, 0, 0), particle_fade )
	else
		particle_is_fade = 0
	end
	if checkbox_rotate:isChecked() then
		particle_is_rotate = 1
		field:addRotationAffector( xcore.x3dpoint(particle_rot_x, particle_rot_y, particle_rot_z), xcore.x3dpoint( 0, 0, 0) )		
	else
		particle_is_rotate = 0
	end
end

function saveSystem()
	-- print("save system")

	-- save to file?
	if editbox_filename:getText() ~= nil then particle_filename = editbox_filename:getText() end
	
	print("saving to file " .. particle_filename)
	
	local file = assert( io.open( path .. particle_filename, "w") )
	file:write("--\n")	
	file:write("-- generated particle system file\n")
	file:write("--\n")

	file:write ( "particle_minpps = ", particle_minpps, "\n" )
	file:write ( "particle_maxpps = ", particle_maxpps, "\n" )
	file:write ( "particle_minlife = ", particle_minlife, "\n" )
	file:write ( "particle_maxlife = ", particle_maxlife, "\n" )

	file:write ( "particle_angle = ", particle_angle, "\n" )
	file:write ( "particle_minsize = ", particle_minsize, "\n" )
	file:write ( "particle_maxsize = ", particle_maxsize, "\n" )
	file:write ( "particle_radius = ", particle_radius, "\n" )
	file:write ( "particle_thickness = ", particle_thickness, "\n" )
	file:write ( "particle_grav = ", particle_grav, "\n" )
	file:write ( "particle_fade = ", particle_fade, "\n" )
	file:write ( "particle_dir_x = ", particle_dir_x, "\n" )
	file:write ( "particle_dir_y = ", particle_dir_y, "\n" )
	file:write ( "particle_dir_z = ", particle_dir_z, "\n" )
	file:write ( "particle_rot_x = ", particle_rot_x, "\n" )
	file:write ( "particle_rot_y = ", particle_rot_y, "\n" )
	file:write ( "particle_rot_z = ", particle_rot_z, "\n" )

	file:write ( "particle_emittertype = ", particle_emittertype, "\n" )
	file:write ( "particle_particletype = ", particle_particletype, "\n" )

	file:write ( "particle_is_grav = ", particle_is_grav, "\n" )
	file:write ( "particle_is_fade = ", particle_is_fade, "\n" )
	file:write ( "particle_is_rotate = ", particle_is_rotate, "\n" )

	file:close()
	save_window:setVisible( false )	
end

function loadSystem()
	-- load file
	
	particle_filename = listbox_filenames:getItem( listbox_filenames:getSelected() )
	
	
	print("loading " .. particle_filename)	
	x = loadfile ( path .. particle_filename)
	x()
	
	setInputs()
	createSystem()
	load_window:setVisible( false )
end

function showSaveDialog()
	-- reset pos & inputs?
	save_window:setVisible( true )
end

function showLoadDialog()
	-- reset pos & inputs?
	listFiles()
	load_window:setVisible( true )
end

function cancel()
	-- clear text field
	-- hide dialog
	save_window:setVisible( false )	
	load_window:setVisible( false )	
	-- reset pos
end

function listFiles()
	listbox_filenames:clear()
	for filename, attr in dirtree( path ) do
		if attr.mode == "file" then
			tmp = string.split( filename, "/" )
			i = table.getn( tmp )
			listbox_filenames:addItem( tmp[i] )
		end
		print(attr.mode, filename)
	end
	listbox_filenames:setSelected( 0 )
end

-- Move the following functions into xlib
function dirtree(dir)
  assert(dir and dir ~= "", "directory parameter is missing or empty")
  if string.sub(dir, -1) == "/" then
    dir=string.sub(dir, 1, -2)
  end
  local diriters = {lfs.dir(dir)}
  local dirs = {dir}
  return function()
    repeat 
      local entry = diriters[#diriters]()
      if entry then 
        if entry ~= "." and entry ~= ".." then 
          local filename = table.concat(dirs, "/").."/"..entry
          local attr = lfs.attributes(filename)
          if attr.mode == "directory" then 
            table.insert(dirs, entry)
            table.insert(diriters, lfs.dir(filename))
          end
          return filename, attr
        end
      else
        table.remove(dirs)
        table.remove(diriters)
      end
    until #diriters==0
  end
end

function string:split(delimiter)
	local result = { }
	local from  = 1
	local delim_from, delim_to = string.find( self, delimiter, from  )
	while delim_from do
		table.insert( result, string.sub( self, from , delim_from-1 ) )
		from  = delim_to + 1
		delim_from, delim_to = string.find( self, delimiter, from  )
	end
	table.insert( result, string.sub( self, from  ) )
	return result
end

-- end of file
