-- zero point war
-- c20011 : lukas irwin
-- visual.lua

VISTAL_TYPE_BLOCK       = 1
VISTAL_TYPE_PHYSICAL    = 2
VISTAL_TYPE_ALERT       = 3
VISTAL_TYPE_SKILLUP     = 4
VISTAL_TYPE_STATUP      = 5
VISTAL_TYPE_LEVELUP     = 6
VISTAL_TYPE_FIRE        = 7

visual = { ID = 0 }

print "[visual]"

-- creation
function visual:new (o)
	-- print "[visual] new visual"
	o = o or {}
	setmetatable(o, self)
	self.__index = self

	o.ID       = 0
	o.life     = 0
	o.time     = 0
	o.endpoint = nil
	o.position = xcore.x3dpoint (0, 0, 0)
	
	return o
end

function visual:update ()
	-- print "[visual] update"
	-- count down life
	self.time = self.time + 1
	-- print ( "self.time:" .. self.time )
	-- print ( "self.life:" .. self.life )

	if self.time >= self.life*2 then
		-- print "[visual] remove"
		return 0
	elseif self.time > (self.life/2) then
		-- print "[visual] stop emitter"
		self.field:setMinParticlesPerSecond( 0 )
		self.field:setMaxParticlesPerSecond( 0 )
	end


	if self.endpoint ~= nil then
		print "move effect"
		self:move()
		-- do this based on tiles?
		if self.position.X == self.endpoint.X and self.position.Z == self.endpoint.Z then
			return 0
		end
	end

	return 1
end

function visual:destroy ()
	-- print "[visual] destroy"
	if self.field ~= nil then
		self.field:destroy()
	end

	if self.text ~= nil then
		self.text:destroy()
	end

end

function visual:setID ( n ) self.ID = n end
function visual:getID () return self.ID end

function visual:setType ( n, r )
	-- print "[visual] setType"
	if n == VISTAL_TYPE_BLOCK then
		-- print "[visual] block"
		-- block sparks
		-- create a particle system
		self.field = sim:addParticleSystem(2, path .. "resources/shield.tga") -- TODO: get creature's blood type
		self.field:setMaterialFlag( xcore.XMF_LIGHTING, false )
		self.field:setMaterialFlag( xcore.XMF_ZWRITE_ENABLE, false )
		self.field:setMaterialType( xcore.XMT_TRANSPARENT_ADD_COLOR )
		-- self.field:setMaterialType( xcore.XMT_TRANSPARENT_VERTEX_ALPHA )
		
		self.field:createPointEmitter( xcore.x3dpoint(0, 0.02, 0), 10, 100,
			xcore.xcolour(255, 255, 255, 255), xcore.xcolour(255, 255, 255, 255),
			10, 100, 360, 10, 25 )

		self.field:addFadeOutAffector( xcore.xcolour(0, 0, 0, 0), 20 )
		-- self.field:addRotationAffector( xcore.x3dpoint(10, 0, 0), xcore.x3dpoint( 0, 0, 0) );
		self.life = 180

	elseif n == VISTAL_TYPE_PHYSICAL then
		-- print "[visual] physical damage"
		-- blood spray
		-- create a particle system
		self.field = sim:addParticleSystem(2, path .. "resources/fireball.bmp") -- TODO: get creature's blood type
		self.field:setMaterialFlag( xcore.XMF_LIGHTING, false )
		self.field:setMaterialFlag( xcore.XMF_ZWRITE_ENABLE, false )
		self.field:setMaterialType( xcore.XMT_TRANSPARENT_ADD_COLOR )
		-- self.field:setMaterialType( xcore.XMT_TRANSPARENT_VERTEX_ALPHA )

		self.field:createPointEmitter( xcore.x3dpoint(0, 0.05, 0), 256, 512,
			xcore.xcolour(255, 255, 255, 255), xcore.xcolour(255, 255, 255, 255),
			600, 1000, 360, 1, 10)

		self.field:addFadeOutAffector( xcore.xcolour(0, 0, 0, 0), 20 )
		-- self.field:addGravityAffector( xcore.x3dpoint(0,-0.03, 0 ), 3000 );
		self.life = 180
	elseif n == VISTAL_TYPE_ALERT then
		print "[visual] Someone has issued an alert"
		-- add a dialog node
		self.text = sim:addTextNode( 1000+self.ID, font_small, "!!!", xcore.xcolour( 255, 255, 255, 0 ) )
		self.life = 60

	elseif n == VISTAL_TYPE_SKILLUP then
		print "[visual] skillup!"
		-- add a dialog node
		self.text = sim:addTextNode( 1000+self.ID, font_small, "Skill +1", xcore.xcolour( 255, 255, 255, 0 ) )
		self.life = 60

	elseif n == VISTAL_TYPE_STATUP then
		print "[visual] Stat Up!"
		-- add a dialog node
		self.text = sim:addTextNode( 1000+self.ID, font_small, "Stat +1", xcore.xcolour( 255, 255, 255, 0 ) )
		self.life = 60

	elseif n == VISTAL_TYPE_LEVELUP then
		print "[visual] Level Up!"
		-- add a dialog node
		self.text = sim:addTextNode( 1000+self.ID, font_small, "Level +1", xcore.xcolour( 255, 255, 255, 0 ) )
		self.life = 60

	elseif n == VISTAL_TYPE_FIRE then
		print "[visual] VISTAL_TYPE_FIRE"
		-- create a particle system
		self.field = sim:addParticleSystem(2, path .. "resources/fireball.bmp") -- TODO: get creature's blood type
		self.field:setMaterialFlag( xcore.XMF_LIGHTING, false )
		self.field:setMaterialFlag( xcore.XMF_ZWRITE_ENABLE, false )
		self.field:setMaterialType( xcore.XMT_TRANSPARENT_ADD_COLOR )

		self.field:createPointEmitter( xcore.x3dpoint(0, 0, 0), 25, 50,
			xcore.xcolour(255, 255, 255, 255), xcore.xcolour(255, 255, 255, 255),
			6000, 8000, 360, 10, 12 )

		self.life = 460
	end
	-- print "[visual] /setType"
end

function visual:getPosition ()
	return self.position
end

function visual:setPosition ( p )
	-- print "[visual] setPosition"
	self.position = p
	if self.field ~= nil then
		self.field:setPosition( xcore.x3dpoint( p.X, p.Y, p.Z ) )
	end

	if self.text ~= nil then
		self.text:setPosition( xcore.x3dpoint( p.X, p.Y, p.Z) )
	end
	-- print "[visual] /setPosition"
end

function visual:setEndPoint ( p )
	print "[visual] setEndPoint"
	self.endpoint = p

	xdif = self.position.X - self.endpoint.X
	ydif = self.position.Z - self.endpoint.Z

	x = xdif
	y = ydif*-1
	
	self.angle = math.atan(y/x)/(math.pi/180)
	if x<0 then
		self.angle = self.angle + 180
	end
	if x>=0 and y<0 then
		self.angle = self.angle + 360
	end

	self.speed = 5
	self.r = self.angle * math.pi/180;
	-- print "[visual] /setEndPoint"
end


function visual:move()
	print "[visual] move"

	local xspeed = math.cos( self.r ) * self.speed
	local yspeed = math.sin( self.r ) * self.speed
	-- print ( "[visual] r:" .. self.r )
	-- print ( "[visual] xspeed:" .. xspeed )
	-- print ( "[visual] xspeed:" .. yspeed )
	-- print ( "[visual] self.position:" .. self.position.X .. ", " .. self.position.Z )
	self:setPosition( xcore.x3dpoint( self.position.X - xspeed, self.position.Y, self.position.Z + yspeed ) )

	-- 

end

