-- Xenos Engine Tutorial Graphics 2 
-- c2007 lukas irwin sysop [ at ] xhrit.com
-- tutorial_graphics_2.lua

print "User Inferface - xenjine Demo"

-- new elements for this tutorial
--  xscrollbar
--  xrect
--  xbutton
--  xlistbox
--  xwindow

-- Required start function. use to set up your program.
function init()
	-- start a simulation
	-- require "module.settings" for video options
	sim = xcore.xsimulation()
	sim:setWindowCaption( "Xenos Engine Tutorial Graphics 2 : User Inferface" )

	-- add a logo
	logo_texture = sim:getTexture( "modules/tutorial_graphics_1/resources/xenos_logo_small.tga" )
	logo_image = sim:addImage( 1001, logo_texture, xcore.x2dpoint(-20, -10),
		xcore.x2drect(0,0,64,64), xcore.xcolour(255,255,255,255) )

	-- add some static text describing the demo
	text = [[
	Xenos Engine Tutorial Graphics 2 ( User Inferface )
	
	This demo will show you the basics of the GUI system
	and how to bind events to actions.
	]]
	sim:addText( 0, text, 20, 20, 800, 800)

	x = 300
	-- add some buttons to create a window and quit
	sim:addButton( 101, xcore.x2drect( 310, 240, 410, 260 ), "New Window" )
	sim:addButton( 102, xcore.x2drect( 310, 280, 410, 300 ), "Quit" )

	-- add a slider to control transparency
	sim:addText( 0, "Transparency Control", 310,20,510,40 )
	scrollbar = sim:addScrollbar(104, true, xcore.x2drect(310, 45, 500, 60))
	scrollbar:setMax(100)
	-- scrollbar:setPos(100)

	-- add a listbox to log the events
	sim:addText( 0, "Logging ListBox", 310,80,510,100 )
	listbox = sim:addListbox(105, true, xcore.x2drect(310, 110, 500, 180))
	
	-- add a checkbox to show background image
	checkbox = sim:addCheckbox( 106, true, "Checkbox", 310, 200, 350, 220 )
	sim:addText( 0, "Checkbox", 335, 204, 400, 220   )

	-- bind events to actions.
	sim:bindkey ( "quit_key_pressed", xcore.XKEY_ESCAPE )
	sim:bindGUI ( "window_button_pressed", 101, xcore.XGUI_BUTTON_CLICKED )	
	sim:bindGUI ( "quit_button_pressed", 102, xcore.XGUI_BUTTON_CLICKED )
	sim:bindGUI ( "scrollbar_changed", 104, xcore.XGUI_SCROLL_BAR_CHANGED )
	sim:bindGUI ( "checkbox_changed", 106, xcore.XGUI_CHECKBOX_CHANGED )

	sim:bindGUI ( "hover_element", 102, xcore.XGUI_ELEMENT_HOVERED )
	sim:bindGUI ( "left_element", 102, xcore.XGUI_ELEMENT_LEFT )

	windows = {}
	buttons = {}
	cnt = 0
	window_id = 1
end

-- Render function; required.
function render()
	-- render simulation tree
	sim:render()
end

-- The required  update function callback is called at a fixed framerate. default is 10/sec; set using sim:setTimestep()
function update()
	sim:update()

	-- while sim:eventcount() > 0 do manualevents() end
	if sim:isKey( "quit_key_pressed" ) then sim:destroy() end
	if sim:isGUI( "quit_button_pressed" ) then sim:destroy() end
	if sim:isGUI( "file_button_pressed" ) then findFile() end
	if sim:isGUI( "window_button_pressed" ) then createWindow() end
	if sim:isGUI( "scrollbar_changed" ) then changeScroll() end
	if sim:isGUI( "checkbox_changed" ) then listbox:addItem( "checkbox changed" ) end
	if sim:isGUI( "hover_element" ) then listbox:addItem( "hover element" ) end
	if sim:isGUI( "left_element" ) then listbox:addItem( "left element" ) end	
	
	for i=1, window_id do
		if sim:isGUI( "button_pressed_"..i ) then destroyWindow( i ) end
	end
end


-- scrollbar will change alpha of gui.
function changeScroll()
	pos = scrollbar:getPos()
	for i=0, xcore.XGDC_COUNT do
		col = sim:getSkinColor( i )
		col.A = pos
		sim:setSkinColor( i, col)
	end
end

-- create a little window with some text on it; also add a string to the list box to log what happened.
function createWindow()
	listbox:addItem( "Window created" )

	cnt =  cnt + 30
	if cnt > 200 then cnt = 0 end
	window_id = window_id+1
	
	windows[window_id] = sim:addWindow( 10, xcore.x2drect(50 + cnt, 50 + cnt, 250 + cnt, 150 + cnt), "Test window")
	buttons[window_id] = sim:addButton( 1000+window_id, xcore.x2drect( 50, 50, 150, 70  ), "Close window" )
	buttons[window_id]:setParent( windows[window_id] )
	
	sim:bindGUI ( "button_pressed_"..window_id, 1000+window_id, xcore.XGUI_BUTTON_CLICKED )
end

function destroyWindow( id )
	listbox:addItem( "Window destroyed" )

	if buttons[ id ] ~= nil then 
		buttons[ id ]:destroy()
		buttons[ id ] = nil
	end

	if windows[ id ] ~= nil then 
		windows[ id ]:destroy()
		windows[ id ] = nil
	end
end

-- end of file