-- Xenos Engine Tutorial Net Client (GUI chat client)
-- c2008-2009 lukas irwin | sysop [ at ] xhrit.com
-- tutorial_chat_client.module

print "[client] Xenos Engine Tutorial Net Client"

-- Required start function. use to set up your program.
function init()
	-- start a simulation
	sim = xcore.xsimulation()
	sim:setWindowCaption( "Xenos Engine Tutorial Net Client" )

	-- start the network subsystem and connect to localhost.
	net = xcore.xnet()
	net:client( "127.0.0.1" )

	-- add a logo
	logo_texture = sim:getTexture( "modules/tutorial_graphics_1/resources/xenos_logo_small.tga" )
	logo_image = sim:addImage( 0, logo_texture, xcore.x2dpoint(-20, -10),
		xcore.x2drect(0,0,64,64), xcore.xcolour(255,255,255,255) )
	
	-- add some static text describing the demo
	text = [[
	Xenos Engine Tutorial Net Client
	
	Provide a username and password to connect
	Type message into input box and press button to send
	Press escape to quit
	]]
	sim:addText( 0, text, 50, 50, 600, 600)
	
	-- add text box (listbox for now), for messege  output
	x = 100
	y = 120
	-- textbox = sim:addTextbox( 0, xcore.x2drect( x,y,x+200,y+200 ) )
	listbox = sim:addListbox( 10, true, xcore.x2drect( x,y,x+300,y+200 ) )
	listbox:setVisible( false )

	-- add edit boxs for input
	x = 100
	y = 300
	editboxmessage = sim:addEditbox( 20,  xcore.x2drect( x,y,x+300,y+20 ) )
	editboxmessage:setVisible( false )

	-- add boxes & labels for username and password
	x = 200
	y = 150
	editboxname = sim:addEditbox( 30,  xcore.x2drect( x,y,x+150,y+20 ) )
	editboxpass = sim:addEditbox( 40,  xcore.x2drect( x,y+50,x+150,y+70 ) )
	editboxname:setVisible( true )
	editboxpass:setVisible( true )

	x = 100
	textname = sim:addText( 2, "name", x, y, x+50, y+20 )
	textpass = sim:addText( 3, "password", x, y+50, x+50, y+70 )

	-- add some buttons
	x = 200
	y = 300
	buttonconnect = sim:addButton( 101, xcore.x2drect( x, y, x+50, y+20 ), "connect" )
	
	x = 300
	y = 300
	buttonsend    = sim:addButton( 102, xcore.x2drect( x, y, x+100, y+20 ), "send" )
	buttonsend:setVisible( false )

	-- bind events to actions.
	sim:bindkey ( "quit_key_pressed", xcore.XKEY_ESCAPE )
	sim:bindGUI ( "connect_button_pressed", 101, xcore.XGUI_BUTTON_CLICKED )
	sim:bindGUI ( "send_button_pressed", 102, xcore.XGUI_BUTTON_CLICKED )

	-- set connection state to 0.
	connected = 0
end

-- Render function; required.
function render()
	-- render simulation tree
	sim:render()
end

-- watch for packets and keyboard/ mouse input.
function update()
	-- update sim
	sim:update()
	-- update network
	net:update()
	-- get oldest message out ov que
	packet = net:get()
	
	if packet ~= nil then
		print "[client] packet not null"
		if tonumber( packet:getType() ) == xcore.XPACKET_CONNECT then
			print "[client] got connected"
		elseif tonumber( packet:getType() ) == xcore.XPACKET_MESSAGE then
			if connected == 1 then
				print "[client] got message"
				-- a = packet:getMessage()
				-- add message to text area.
				listbox:addItem( packet:getMessage() )
			end
		elseif tonumber( packet:getType() ) == xcore.XPACKET_AUTH then
			print "[client] got auth"
			a = packet:getMessage()
			if a == "allow" then
				connected = 1
				-- if auth accept then hide un / pw boxes and show chat stuff && connected = 1
				print "[client] auth allowed by server"
				editboxmessage:setVisible( true )
				listbox:setVisible( true )
				buttonsend:setVisible( true )
				buttonconnect:setVisible( false )
				editboxname:setVisible( false )
				editboxpass:setVisible( false )
				textname:setVisible( false )
				textpass:setVisible( false )
			else
				-- TODO: show rejection message
				print "[client] auth denied by server"
				connected = 0
			end
		else
			print ( "[client] unknown packet type: " .. packet:getType() )
		end

	end

	-- watch for events
	if sim:isKey( "quit_key_pressed" ) then
		-- exit program
		sim:destroy()
		xcore.destroy()
	end
	if sim:isGUI( "send_button_pressed" ) then sendMessage() end
	if sim:isGUI( "connect_button_pressed" ) then doConnect() end
end

-- send un and pw to server
function doConnect()
	print "[client] doConnect"
	n = editboxname:getText() -- get name from xeditbox
	p = editboxpass:getText() -- get password from xeditbox
	t = n .. ":" .. p         -- glue them together
	net:send( xcore.XPACKET_VALIDATE, t ) -- send the validation string
end

-- send text message to server
function sendMessage()
	print "[client] send message"
	-- get text from xeditbox
	t = editboxmessage:getText()
	if t ~= nil then
		-- send via xnet object
		net:send( xcore.XPACKET_MESSAGE, t )
		-- clear xeditbox
		editboxmessage:setText("")
	end
end

-- end of file