-- Game Enemy AI Behavior - Run Away & Follow Player
-- Philip Radvan @ http://www.freeactionscript.com
-- ported to xenos / lua by Lukas Irwin
-- 

path = "modules/tutorial_steering_behaviors_1/"
loadfile ( path .. "enemy.lua" )()

-- required
function init()
	print "[module] loading module tutorial steering behaviors 1"
	width = 800
	height = 600
	sim = xcore.xsimulation ( xcore.XDT_OPENGL , width, height, 16, false, false, false )
	sim:setWindowCaption( "Steering Behaviors 1" )

	-- add some static text describing the demo
	text = [[
	Steering Behaviors 1

	Arrow keys move
	Press escape to quit
	]]
	sim:addText( 0, text, 50, 50, 600, 600)

	-- add a static camera
	sim:addCamera ( 0, xcore.CAMERA_STATIC, 0,200,0,  0,0,0)

	math.randomseed(os.time())
	enemiesArray = {}
	radians = 180 / math.pi

	-- create a player
	player_mesh = sim:addCubeMesh ( 1 )
	player_mesh:addTexture ( path .. "p.jpg", 0 )
	player_mesh:setScale ( xcore.x3dpoint(1, 0, 1) )
	player_mesh:setMaterialFlag( xcore.XMF_LIGHTING, false )
	player_mesh:setPosition( xcore.x3dpoint( 0,0, 0  ) )

	-- create some enemies
	createEnemies( 5, "typeA", "e1.jpg" )
	createEnemies( 5, "typeB", "e2.jpg" )
	createEnemies( 5, "typeC", "e3.jpg" )

	-- make it so you can exit the demo
	sim:bindkey ( "quit",  xcore.XKEY_ESCAPE  )

	-- movement keys
	sim:bindkey ( "move_left",  xcore.XKEY_LEFT  )
	sim:bindkey ( "move_right", xcore.XKEY_RIGHT )
	sim:bindkey ( "move_forward",    xcore.XKEY_UP )
	sim:bindkey ( "move_back",  xcore.XKEY_DOWN  )
end

-- required
function render()
	-- render simulation tree
	sim:render()
end

-- required
function update()
	-- move player based on kb input (todo: mouse!)
	p = player_mesh:getPosition()
	if sim:isKey( "move_left" ) then 
		if p.Z < 180 then p = xcore.x3dpoint( p.X, p.Y, p.Z+0.3 ) end
	end
	if sim:isKey( "move_right" ) then
		if p.Z > -180 then p = xcore.x3dpoint( p.X, p.Y, p.Z-0.3 )	end
	end
	if sim:isKey( "move_forward" ) then
		if p.X < 130 then p = xcore.x3dpoint( p.X+0.3, p.Y, p.Z ) end
	end
	if sim:isKey( "move_back" ) then
		if p.X > -130 then p = xcore.x3dpoint( p.X-0.3, p.Y, p.Z ) end
	end
	player_mesh:setPosition( p )

	-- update enemy positions
	updateEnemies()

	-- test for escape key and then update sim
	if sim:isKey( "quit" ) then sim:destroy() end
	sim:update()	
end

--  create enemies
function createEnemies( enemyAmount, enemyBehavior, enemyGraphic ) -- int number of enemies, text behavior, text graphic image
	-- run a for loop based on the amount of enemies
	for i = 1, enemyAmount do
		-- add new enemy to array
		p = table.getn( enemiesArray ) + 1
		enemiesArray[ p ] = enemy:new()

		enemiesArray[ p ].model = sim:addCubeMesh ( 1 )
		enemiesArray[ p ].model:addTexture ( path .. enemyGraphic, 0 )
		enemiesArray[ p ].model:setMaterialFlag( xcore.XMF_LIGHTING, false )
		enemiesArray[ p ].model:setScale( xcore.x3dpoint(1, 0, 1) )
		-- give new enemy a random x/y position based on stage width/height
		enemiesArray[ p ]._x = math.random( -150, 150 )
		enemiesArray[ p ]._y = math.random( -150, 150 )
		enemiesArray[ p ]._rotation = 0 -- math.random(0, 360)
	
		-- set enemy behavior
		if enemyBehavior == "typeA" then
			-- define enemy characteristics
			enemiesArray[ p ].speed = 0.1
			enemiesArray[ p ].turnRate = .001
			enemiesArray[ p ].agroRange = 100
			enemiesArray[ p ].mode = "follow"
		elseif enemyBehavior == "typeB" then
			-- define enemy characteristics
			enemiesArray[ p ].speed = 0.04
			enemiesArray[ p ].turnRate = .0005
			enemiesArray[ p ].agroRange = 200
			enemiesArray[ p ].mode = "follow"
		elseif enemyBehavior == "typeC" then
			-- define enemy characteristics
			enemiesArray[ p ].speed = 0.05
			enemiesArray[ p ].turnRate = .0002
			enemiesArray[ p ].agroRange = 100
			enemiesArray[ p ].mode = "run"
		end
	end
end

-- Update enemies
function updateEnemies()
	-- run a for loop based on the amount of enemies
	for i = 1, table.getn( enemiesArray ) do
		-- run follow function with enemy as the follower
		enemiesArray[i]:update()		
		updatePosition( enemiesArray[i] , player_mesh:getPosition())
	end
end

-- Update an enemy position
function updatePosition(follower, target) -- enemy object, x3dpoint of targets position
	-- calculate distance between follower and target
	follower.distanceX = target.X-follower._x
	follower.distanceY = target.Z-follower._y
	
	-- get total distance as one number
	follower.distanceTotal = math.sqrt(follower.distanceX * follower.distanceX + follower.distanceY * follower.distanceY)
	
	-- check if target is within agro range
	if follower.distanceTotal <= follower.agroRange then
		-- calculate how much to move
		follower.moveDistanceX = follower.turnRate * follower.distanceX / follower.distanceTotal
		follower.moveDistanceY = follower.turnRate * follower.distanceY / follower.distanceTotal

		-- increase current speed
		follower.moveX = follower.moveX + follower.moveDistanceX
		follower.moveY = follower.moveY + follower.moveDistanceY

		-- get total move distance
		follower.totalmove = math.sqrt(follower.moveX * follower.moveX + follower.moveY * follower.moveY)

		-- apply easing
		follower.moveX = follower.speed * follower.moveX / follower.totalmove
		follower.moveY = follower.speed * follower.moveY / follower.totalmove

		-- move & rotate follower
		if follower.mode == "follow" then
			follower._x = follower._x + follower.moveX
			follower._y = follower._y + follower.moveY
			follower._rotation = math.abs( ( math.atan2(follower.moveY, follower.moveX) * radians ) -180 ) +90
		elseif follower.mode == "run" then
			follower._x = follower._x - follower.moveX
			follower._y = follower._y - follower.moveY
			follower._rotation = ( math.abs( ( math.atan2(follower.moveY, follower.moveX) * radians ) -180 ) +90 ) -180
		end
	end
end

-- end of file